Public Class cCallEvent

    Public Const IN_BOUND_CALL_REJECT As Integer = 0
    Public Const IN_BOUND_CALL_CONNECT_USER As Integer = 1
    Public Const IN_BOUND_CALL_PUT_IN_QUEUE As Integer = 2

    Public Sub OnIncomingCallFromLine(ByRef objCallData As cCallData, ByRef objLineData As cLineData, ByVal sDialNo As String)

        If objLineData.m_nCallType = IN_BOUND_CALL_REJECT Then
            cServerSIP.m_objServerSIP.CloseCall(objCallData)
            Exit Sub
        End If

        If objLineData.m_nCallType = IN_BOUND_CALL_CONNECT_USER Then
            cServerSIP.m_objServerSIP.DialCallToUser(objCallData, objLineData.m_sCallTo)
            Exit Sub
        End If

        If objLineData.m_nCallType = IN_BOUND_CALL_PUT_IN_QUEUE Then

            Dim objQueueData As cQueueData = cServerSIP.m_objServerSIP.m_objQueueList.FindQueue(objLineData.m_sCallTo)

            If objQueueData Is Nothing Then
                cServerSIP.m_objServerSIP.CloseCall(objCallData)
            Else
                objCallData.m_nCallType = cServerSIP.CALL_TYPE_QUEUE
                OnIncomingCallQueueType(objCallData, objQueueData.m_unQueueId)
            End If

        End If

    End Sub

    Public Sub OnIncomingCallFromUser(ByRef objUserCallData As cCallData, ByRef objUserData As cUserData, ByVal sDialNo As String)

        If objUserData.m_bStealthListener = True Then

            objUserCallData.m_nCallType = cServerSIP.CALL_TYPE_STEALTH
            OnIncomingCallStealthType(objUserCallData, objUserData, sDialNo)

        Else

            objUserCallData.m_nCallType = cServerSIP.CALL_TYPE_REGULAR
            OnIncomingCallRegularType(objUserCallData, objUserData, sDialNo)

        End If

    End Sub

    Public Sub OnIncomingCallQueueType(ByRef objCallData As cCallData, ByVal unQueueId As UInteger)

        objCallData.m_unQueueId = unQueueId
        cServerSIP.m_objServerSIP.AcceptCall(objCallData)

    End Sub

    Public Sub OnIncomingCallStealthType(ByRef objCallData As cCallData, ByRef objUserData As cUserData, ByVal sDialNo As String)

        objCallData.m_sStealthDialNo = sDialNo.ToLower()

        If cServerSIP.m_objServerSIP.AcceptCall(objCallData) = True Then
            objUserData.m_nCallCount += 1
        End If


    End Sub

    Public Sub OnIncomingCallRegularType(ByRef objCallData As cCallData, ByRef objUserData As cUserData, ByVal sDialNo As String)

        Dim nDialPlanIndex As Integer = cServerSIP.m_objServerSIP.m_objDialPlanList.SearchDialPlan(sDialNo)

        If nDialPlanIndex <> -1 Then

            Dim sLineName As String = ""
            sDialNo = cServerSIP.m_objServerSIP.m_objDialPlanList.ApplyDialPlan(nDialPlanIndex, sDialNo, sLineName)

            If Not cServerSIP.m_objServerSIP.DialCallToLine(objCallData, sDialNo, sLineName) Is Nothing Then
                objUserData.m_nCallCount += 1
            End If

        Else

            If Not cServerSIP.m_objServerSIP.DialCallToUser(objCallData, sDialNo) Is Nothing Then
                objUserData.m_nCallCount += 1
            End If

        End If

    End Sub

    '////////////////////////////////////////////////////

    Public Sub OnAcceptIncomingCallConnected(ByRef objCallData As cCallData)

        objCallData.m_bConnected = True

        Select Case objCallData.m_nCallType

            Case cServerSIP.CALL_TYPE_STEALTH
                OnConnectedCallStealthType(objCallData)

            Case cServerSIP.CALL_TYPE_QUEUE
                OnConnectedCallQueueType(objCallData)

            Case cServerSIP.CALL_TYPE_REGULAR

        End Select

    End Sub

    Public Sub OnDialCallConnected(ByRef objCallData As cCallData)

        objCallData.m_bConnected = True

        Select Case (objCallData.m_nCallType)

            Case cServerSIP.CALL_TYPE_REGULAR
                OnConnectedCallRegularType(objCallData)

            Case cServerSIP.CALL_TYPE_QUEUE

            Case cServerSIP.CALL_TYPE_STEALTH

        End Select

    End Sub

    '////////////////////////////////////////////////////////

    Public Sub OnConnectedCallRegularType(ByRef objCallData As cCallData)

        Dim objBindCallData As cCallData = objCallData.m_objBindCallData

        If (Not objBindCallData Is Nothing) And (objBindCallData.m_bConnected = False) Then

            If cServerSIP.m_objServerSIP.AcceptCall(objBindCallData) = False Then
                cServerSIP.m_objServerSIP.CloseCall(objBindCallData)
                cServerSIP.m_objServerSIP.CloseCall(objCallData)
                Exit Sub
            End If

        End If

        cServerSIP.m_objServerSIP.StartVoiceStream(objCallData, objBindCallData)

        cServerSIP.m_objServerSIP.m_objStealthList.TryCaptureCall(objCallData)
        cServerSIP.m_objServerSIP.m_objStealthList.TryCaptureCall(objBindCallData)

    End Sub

    Public Sub OnConnectedCallStealthType(ByRef objCallData As cCallData)

        cServerSIP.m_objServerSIP.m_objStealthList.AddCallToStealthList(objCallData)

        Dim nStealthSessionId As Integer = cServerSIP.m_objServerSIP.m_objServerData.GetStealthSessionId()
        cServerSIP.m_objServerSIP.AddCallToSession(objCallData, nStealthSessionId)

    End Sub

    Public Sub OnConnectedCallQueueType(ByRef objCallData As cCallData)
        cServerSIP.m_objServerSIP.m_objQueueList.AddCallToQueue(objCallData)
    End Sub

    '////////////////////////////////////////////////////////

    Public Sub OnDialCallTimeOut(ByRef objCallData As cCallData)

        Dim objBindCallData As cCallData = objCallData.m_objBindCallData

        If Not objBindCallData Is Nothing Then
            cServerSIP.m_objServerSIP.SendResponse(objBindCallData, 408, "Request Timeout")
            cServerSIP.m_objServerSIP.CloseCall(objBindCallData)
        End If

        cServerSIP.m_objServerSIP.CloseCall(objCallData)

    End Sub

    Public Sub OnAcceptCallTimeOut(ByRef objCallData As cCallData)
        OnDisconnectCall(objCallData)
    End Sub

    Public Sub OnDisconnectCall(ByRef objCallData As cCallData)

        Dim objBindCallData As cCallData = objCallData.m_objBindCallData

        If Not objBindCallData Is Nothing Then
            cServerSIP.m_objServerSIP.CloseCall(objBindCallData)
        End If

        cServerSIP.m_objServerSIP.CloseCall(objCallData)

    End Sub

    Public Sub OnIncomingCallCancel(ByRef objCallData As cCallData)
        OnDisconnectCall(objCallData)
    End Sub

    Public Sub OnSessionLost(ByRef objCallData As cCallData)
        OnDisconnectCall(objCallData)
    End Sub

End Class
