Imports System.Diagnostics.PerformanceCounter

Public Class cServerSIP

    Private m_objUsageCPU As New PerformanceCounter("Processor", "% Processor Time", "_Total")
    'Private m_objAvailMEM As New PerformanceCounter("Memory", "Available MBytes")

    Public Const CALL_TYPE_REGULAR As Integer = 0
    Public Const CALL_TYPE_QUEUE As Integer = 1
    Public Const CALL_TYPE_STEALTH As Integer = 2

    Public Const PLAY_WAVE_NONE_TYPE As Integer = 100
    Public Const PLAY_WAVE_DONE_TYPE As Integer = 101
    Public Const PLAY_WAVE_LOOP_TYPE As Integer = 102

    Public Const QUEUE_TICK_ID As Integer = 2001

    Public Shared m_objServerSIP As cServerSIP
    Private m_objVaxTeleCOM As New cVaxTeleCOM

    Public m_objCallList As New cCallList
    Public m_objLineList As New cLineList
    Public m_objDialPlanList As New cDialPlanList
    Public m_objQueueList As New cQueueList
    Public m_objUserList As New cUserList
    Public m_objStealthList As New cStealthList

    Public m_objServerData As New cServerData
    Public m_objErrorLog As New cErrorLog

    Private m_objClientRegEvent As New cClientRegisterEvent
    Private m_objCallEvent As New cCallEvent
    Private m_objResponseEvent As New cResponseEvent
    Private m_objHoldCallEvent As New cHoldCallEvent
    Private m_objTransferCallEvent As New cTransferCallEvent
    Private m_objLineRegisterEvent As New cLineRegisterEvent

    Public m_objPlayWaveList As New Hashtable

    Private m_bStatusRun As Boolean

    Public Const QUEUE_CALL_TYPE As Integer = 100
    Public Const STEALTH_CALL_TYPE As Integer = 101
    Public Const BLIND_TRANSFER_CALL_TYPE As Integer = 102

    Sub New()
        m_objServerSIP = Me
        m_bStatusRun = False
    End Sub

    Public Function StartServerSIP() As Boolean

        If m_bStatusRun = False Then
            m_bStatusRun = InitVaxTeleSIP()
        End If

        Dim objParamList As New ArrayList

        If m_bStatusRun = True Then
            objParamList.Add("SIP Server is running.")
            cTelnetServer.m_objTelnetServer.SendCommandToAll("StatusServerSIP", objParamList)
        Else
            objParamList.Add("Unable to start SIP Server (see error log).")
            cTelnetServer.m_objTelnetServer.SendCommandToAll("StatusServerSIP", objParamList)
        End If

        StartServerSIP = m_bStatusRun

    End Function

    Public Sub StopServerSIP()

        If m_bStatusRun = True Then
            UnInitVaxTeleSIP()
            m_bStatusRun = False
        End If


        Dim objParamList As New ArrayList
        objParamList.Add("SIP Server is not running.")

        cTelnetServer.m_objTelnetServer.SendCommandToAll("StatusServerSIP", objParamList)

    End Sub

    Private Sub UnInitVaxTeleSIP()

        LogStatus(" ")

        m_objCallList.DeleteAll()
        m_objServerData.UnLoadServer()

        LogStatus("Uninitializing VaxTele SIP Server.")
        m_objVaxTeleCOM.UnInitialize()
        LogStatus("VaxTele SIP Server uninitialized successfully.")

        m_objUserList.DeleteAll()
        LogStatus("VaxTele SIP Server stopped successfully.")

    End Sub

    Private Function InitVaxTeleSIP() As Boolean

        LogStatus(" ")

        Dim objReg As New cRegistry
        If objReg.GetStartServer() = False Then
            InitVaxTeleSIP = False
            Exit Function
        End If

        Dim sListenIP As String = ""
        Dim sDomainRealm As String = ""
        Dim sListenPort As String = ""

        objReg.GetComm(sListenIP, sListenPort, sDomainRealm)

        m_objClientRegEvent.SetDomainRealm(sDomainRealm)
        m_objVaxTeleCOM.SetLicenseKey("TRIAL-LICENSE-KEY")

        LogStatus("Initializing VaxTele SIP Server.")

        If m_objVaxTeleCOM.Initialize(sDomainRealm, sListenIP, Convert.ToInt32(sListenPort), sListenIP, -1) = False Then
            LogStatus("Unable to initialize VaxTele SIP Server (see error log).")
            LogError("")

            InitVaxTeleSIP = False
            Exit Function
        End If

        LogStatus("VaxTele SIP Server initialized successfully.")
        m_objServerData.LoadServer()
        LogStatus("VaxTele SIP Server started successfully.")

        InitVaxTeleSIP = True


    End Function

    Public Sub BindTwoCall(ByRef objCallDataA As cCallData, ByRef objCallDataB As cCallData)
        objCallDataA.m_objBindCallData = objCallDataB
        objCallDataB.m_objBindCallData = objCallDataA
    End Sub

    Public Function SendResponse(ByRef objCallData As cCallData, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String) As Boolean

        If objCallData.m_bConnected = True Then
            SendResponse = False
            Exit Function
        End If

        SendResponse = m_objVaxTeleCOM.SendResponse(objCallData.m_nCallId, nStatusCode, sReasonPhrase, "")

    End Function

    Public Sub CloseCall(ByRef objCallData As cCallData)

        Select Case (objCallData.m_nCallType)

            Case CALL_TYPE_QUEUE
                m_objQueueList.RemoveQueueCall(objCallData)

            Case CALL_TYPE_STEALTH
                m_objStealthList.RemoveStealthCall(objCallData)

        End Select

        PlayWaveStopToCall(objCallData)
        m_objStealthList.FreeCaptureCall(objCallData)

        CloseSession(objCallData)
        m_objVaxTeleCOM.CloseCall(objCallData.m_nCallId)

        Dim objUserData As cUserData = m_objUserList.FindUser(objCallData.m_unUserId)

        If Not objUserData Is Nothing Then
            objUserData.m_nCallCount -= 1
        End If

        m_objCallList.RemoveCall(objCallData)

    End Sub

    Public Function DialCallToLine(ByRef objFromCallData As cCallData, ByVal sDialNo As String, ByVal sLineName As String) As cCallData

        Dim objLineData As cLineData = m_objLineList.FindLine(sLineName)

        If objLineData Is Nothing Then
            SendResponse(objFromCallData, 404, "User Not Found")
            CloseCall(objFromCallData)

            DialCallToLine = Nothing
            Exit Function
        End If

        Dim nDialCall As Integer = m_objVaxTeleCOM.DialCallToLine(objLineData.m_nLineId, sDialNo, objLineData.m_sCodecList, 120)
        If nDialCall = -1 Then
            SendResponse(objFromCallData, 404, "User Not Found")
            CloseCall(objFromCallData)

            DialCallToLine = Nothing
            Exit Function
        End If

        Dim objDialCallData As cCallData = m_objCallList.AddCall(nDialCall)
        BindTwoCall(objFromCallData, objDialCallData)

        DialCallToLine = objDialCallData

    End Function

    Public Function DialCallToUser(ByRef objFromCallData As cCallData, ByVal sToUserId As String) As cCallData

        Dim objUserData As cUserData = m_objUserList.FindUser(sToUserId)

        If objUserData Is Nothing Then
            SendResponse(objFromCallData, 404, "User Not Found")
            CloseCall(objFromCallData)

            DialCallToUser = Nothing
            Exit Function
        End If

        Dim nDialCall As Integer = m_objVaxTeleCOM.DialCallToUser(objFromCallData.m_sCallerId, sToUserId, objUserData.m_sCodecList, 120)
        If nDialCall = -1 Then
            SendResponse(objFromCallData, 404, "User Not Found")
            CloseCall(objFromCallData)

            DialCallToUser = Nothing
            Exit Function
        End If

        objUserData.m_nCallCount += 1

        Dim objDialCallData As cCallData = m_objCallList.AddCall(nDialCall)

        objDialCallData.m_sUserId = sToUserId
        objDialCallData.m_unUserId = objUserData.m_unUserId

        BindTwoCall(objFromCallData, objDialCallData)

        DialCallToUser = objDialCallData

    End Function

    Public Function AcceptCall(ByRef objCallData As cCallData) As Boolean
        Dim bResult As Boolean = m_objVaxTeleCOM.AcceptCall(objCallData.m_nCallId, objCallData.m_sCodec, 120)

        If bResult = False Then
            LogError("")
            LogStatus("cServerSIP.AcceptCall() method fails.")
        End If

        AcceptCall = bResult
    End Function

    Public Sub CreateSession(ByRef objCallData As cCallData)

        CloseSession(objCallData)

        objCallData.m_nSessionId = m_objVaxTeleCOM.CreateSession()
        m_objVaxTeleCOM.AddCallToSession(objCallData.m_nCallId, objCallData.m_nSessionId)

    End Sub

    Public Sub AddCallToSession(ByRef objCallData As cCallData, ByVal nSessionId As Integer)
        m_objVaxTeleCOM.AddCallToSession(objCallData.m_nCallId, nSessionId)
    End Sub

    Public Sub CloseSession(ByRef objCallData As cCallData)

        If objCallData.m_nSessionId = -1 Then
            Exit Sub
        End If

        PlayWaveStopToCall(objCallData)

        m_objVaxTeleCOM.CloseSession(objCallData.m_nSessionId)
        objCallData.m_nSessionId = -1

    End Sub

    Public Sub TransferCallAccept(ByRef objCallData As cCallData)
        m_objVaxTeleCOM.TransferCallAccept(objCallData.m_nCallId)
    End Sub

    Public Sub DisableDetectDTMF(ByRef objCallData As cCallData)

        objCallData.m_bDigitDetect = False
        objCallData.m_nDigitPRESS = 0
        objCallData.m_sDigitPRESS = ""

        m_objVaxTeleCOM.DetectDTMF(0, objCallData.m_nSessionId, False) 'Disable RFC2833 DTMF detection

    End Sub

    Public Sub EnableDetectDTMF(ByRef objCallData As cCallData)

        objCallData.m_bDigitDetect = True
        objCallData.m_nDigitPRESS = 0
        objCallData.m_sDigitPRESS = ""

        'Enable RFC2833 DTMF detection

        If m_objVaxTeleCOM.DetectDTMF(0, objCallData.m_nSessionId, True) = False Then
            LogError("")
        End If

    End Sub

    Public Function RejectClientRegister(ByVal nRegId As Integer) As Boolean
        RejectClientRegister = m_objVaxTeleCOM.RejectClientRegister(nRegId)
    End Function

    Public Function AuthClientRegister(ByVal nRegId As Integer) As Boolean
        AuthClientRegister = m_objVaxTeleCOM.AuthClientRegister(nRegId)
    End Function

    Public Sub OnClientUnRegister(ByVal sUserLogin As String)
        m_objClientRegEvent.OnClientUnRegister(sUserLogin)
    End Sub

    Public Sub OnClientRegister(ByVal sUserLogin As String, ByVal sDomain As String, ByVal nRegId As Integer)
        m_objClientRegEvent.OnClientRegister(sUserLogin, sDomain, nRegId)
    End Sub

    Public Sub OnClientRegisterSuccess(ByVal sUserLogin As String)
        m_objClientRegEvent.OnClientRegisterSuccess(sUserLogin)
    End Sub

    Public Sub OnClientRegisterFail(ByVal sUserLogin As String)
        m_objClientRegEvent.OnClientRegisterFail(sUserLogin)
    End Sub

    Public Sub OnIncomingCallAnonymous(ByVal nCallId As Integer, ByVal sCallFrom As String, ByVal sCallTo As String, ByVal sFromURI As String, ByVal sToURI As String, ByVal sFromIP As String, ByVal nFromPort As Integer)
        m_objVaxTeleCOM.SendResponse(nCallId, 404, "Anonymous not allowed.", "")
    End Sub

    Public Sub OnIncomingCallFromLine(ByVal nCallId As Integer, ByVal nLineId As Integer, ByVal sCallFrom As String, ByVal sCallTo As String, ByVal sFromURI As String, ByVal sToURI As String, ByVal sFromIP As String, ByVal nFromPort As Integer)

        Dim objLineData As cLineData = m_objLineList.FindLine(nLineId)

        If objLineData Is Nothing Then
            m_objVaxTeleCOM.CloseCall(nCallId)
            Exit Sub
        End If

        Dim objCallData As cCallData = m_objCallList.AddCall(nCallId)

        objCallData.m_sCodec = objLineData.m_sCodecList
        objCallData.m_sCallerId = sCallFrom

        CreateSession(objCallData)

        m_objCallEvent.OnIncomingCallFromLine(objCallData, objLineData, sCallTo)

    End Sub

    Public Sub OnIncomingCallFromUser(ByVal nCallId As Integer, ByVal sUserName As String, ByVal sCallFrom As String, ByVal sCallTo As String, ByVal sFromURI As String, ByVal sToURI As String, ByVal sFromIP As String, ByVal nFromPort As Integer)

        Dim objUserData As cUserData = m_objUserList.FindUser(sUserName)

        If objUserData Is Nothing Then
            Exit Sub
        End If

        Dim objCallData As cCallData = m_objCallList.AddCall(nCallId)

        objCallData.m_sCodec = objUserData.m_sCodecList
        objCallData.m_unUserId = objUserData.m_unUserId
        objCallData.m_sUserId = sUserName
        objCallData.m_sCallerId = sCallFrom

        CreateSession(objCallData)

        m_objVaxTeleCOM.EnableDialTone(nCallId, m_objServerData.GetDialToneSessionId())

        m_objCallEvent.OnIncomingCallFromUser(objCallData, objUserData, sCallTo)

    End Sub

    Public Sub StartVoiceStream(ByRef objCallDataA As cCallData, ByRef objCallDataB As cCallData)

        CloseSession(objCallDataA)
        CloseSession(objCallDataB)

        Dim nSessionId As Integer = m_objVaxTeleCOM.CreateSession()

        objCallDataA.m_nSessionId = nSessionId
        objCallDataB.m_nSessionId = nSessionId

        AddCallToSession(objCallDataA, nSessionId)
        AddCallToSession(objCallDataB, nSessionId)

    End Sub

    Public Sub OnAcceptIncomingCallConnected(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objCallEvent.OnAcceptIncomingCallConnected(objCallData)

    End Sub

    Public Sub OnDialCallConnected(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objCallEvent.OnDialCallConnected(objCallData)

    End Sub

    Public Sub OnDialCallTimeOut(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objCallEvent.OnDialCallTimeOut(objCallData)

    End Sub

    Public Sub OnAcceptCallTimeOut(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objCallEvent.OnAcceptCallTimeOut(objCallData)

    End Sub

    Public Sub OnDisconnectCall(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objCallEvent.OnDisconnectCall(objCallData)

    End Sub

    Public Sub OnIncomingCallCancel(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objCallEvent.OnIncomingCallCancel(objCallData)

    End Sub

    Public Sub OnSessionLost(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objCallEvent.OnSessionLost(objCallData)

    End Sub

    Public Sub OnProvisionalResponse(ByVal nCallId As Integer, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objResponseEvent.OnProvisionalResponse(objCallData, nStatusCode, sReasonPhrase)

    End Sub

    Public Sub OnFailureResponse(ByVal nCallId As Integer, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objResponseEvent.OnFailureResponse(objCallData, nStatusCode, sReasonPhrase)

    End Sub

    Public Sub OnRedirectionResponse(ByVal nCallId As Integer, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String, ByVal sContact As String)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objResponseEvent.OnRedirectionResponse(objCallData, nStatusCode, sReasonPhrase, sContact)

    End Sub

    Public Sub OnDigitDetectDTMF(ByVal nSessionId As Integer, ByVal nCallId As Integer, ByVal nDigit As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        Dim sDigit As String = "0123456789*#ABCDEF"

        m_objQueueList.OnPressDigitDTMF(objCallData, Convert.ToString(sDigit(nDigit)))

    End Sub

    Public Sub OnHoldCall(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objHoldCallEvent.OnHoldCall(objCallData)

    End Sub

    Public Sub OffHoldCall(ByVal nCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objHoldCallEvent.OffHoldCall(objCallData)

    End Sub

    Public Sub OnTransferCallBlind(ByVal nCallId As Integer, ByVal sTransferee As String, ByVal sTransferTo As String, ByVal sTransferToURI As String)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objTransferCallEvent.OnTransferCallBlind(objCallData, sTransferee, sTransferTo, sTransferToURI)

    End Sub

    Public Sub OnTransferCallAttend(ByVal nCallId As Integer, ByVal nTransferToCallId As Integer)

        Dim objCallData As cCallData = m_objCallList.FindCall(nCallId)
        If objCallData Is Nothing Then
            Exit Sub
        End If

        Dim objTransferToCallData As cCallData = m_objCallList.FindCall(nTransferToCallId)
        If objTransferToCallData Is Nothing Then
            Exit Sub
        End If

        m_objTransferCallEvent.OnTransferCallAttend(objCallData, objTransferToCallData)

    End Sub

    Public Function PlayWaveStopToCall(ByRef objCallData As cCallData) As Boolean

        If objCallData.m_nPlayWaveType = PLAY_WAVE_NONE_TYPE Then
            PlayWaveStopToCall = True
            Exit Function
        End If

        If objCallData.m_nPlayWaveType = PLAY_WAVE_DONE_TYPE Then
            m_objPlayWaveList.Remove(objCallData.m_nSessionId)
        End If

        objCallData.m_nPlayWaveType = PLAY_WAVE_NONE_TYPE

        PlayWaveStopToCall = m_objVaxTeleCOM.PlayWaveStopToSession(objCallData.m_nSessionId)

    End Function

    Public Function PlayWaveStartToCall(ByRef objCallData As cCallData, ByVal nPlayWaveId As Integer, ByVal bLoopPlay As Boolean) As Boolean

        If bLoopPlay = False Then
            objCallData.m_nPlayWaveType = PLAY_WAVE_DONE_TYPE
            m_objPlayWaveList.Add(objCallData.m_nSessionId, objCallData)
        Else
            objCallData.m_nPlayWaveType = PLAY_WAVE_LOOP_TYPE
        End If

        PlayWaveStartToCall = m_objVaxTeleCOM.PlayWaveStartToSession(nPlayWaveId, objCallData.m_nSessionId, bLoopPlay)

    End Function

    Public Sub OnPlayWaveToSessionDone(ByVal nPlayWaveId As Integer, ByVal nSessionId As Integer)

        Dim objCallData As cCallData = m_objPlayWaveList.Item(nSessionId)

        If objCallData Is Nothing Then
            Exit Sub
        End If

        m_objPlayWaveList.Remove(nSessionId)
        objCallData.m_nPlayWaveType = PLAY_WAVE_NONE_TYPE

        m_objQueueList.OnPlayWaveDoneToCall(objCallData)

    End Sub

    Public Sub OnLineRegisterTrying(ByVal nLineId As Integer)
        m_objLineRegisterEvent.OnLineRegisterTrying(nLineId)
    End Sub

    Public Sub OnLineRegisterFail(ByVal nLineId As Integer, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String)
        m_objLineRegisterEvent.OnLineRegisterFail(nLineId, nStatusCode, sReasonPhrase)
    End Sub

    Public Sub OnLineRegisterSuccess(ByVal nLineId As Integer)
        m_objLineRegisterEvent.OnLineRegisterSuccess(nLineId)
    End Sub

    Public Sub OnLineUnRegisterTrying(ByVal nLineId As Integer)
        m_objLineRegisterEvent.OnLineUnRegisterTrying(nLineId)
    End Sub

    Public Sub OnLineUnRegisterFail(ByVal nLineId As Integer, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String)
        m_objLineRegisterEvent.OnLineUnRegisterFail(nLineId, nStatusCode, sReasonPhrase)
    End Sub

    Public Sub OnLineUnRegisterSuccess(ByVal nLineId As Integer)
        m_objLineRegisterEvent.OnLineUnRegisterSuccess(nLineId)
    End Sub

    Public Sub OnVaxTeleTick(ByVal nTickId As Integer)

        If nTickId = QUEUE_TICK_ID Then
            m_objQueueList.ProcessQueueTick()
        End If

    End Sub

    Public Sub LogError(ByVal sError As String)
        m_objErrorLog.LogError(sError)
    End Sub

    Public Sub LogStatus(ByVal sStatus As String)

        Dim objParamList As New ArrayList
        objParamList.Add(sStatus)

        cTelnetServer.m_objTelnetServer.SendCommandToAll("StatusLog", objParamList)

    End Sub

    Public Function LoadWaveFileToSession(ByVal sFileName As String, ByRef nPlayWaveId As Integer, ByRef nSessionId As Integer) As Boolean

        LogStatus("Loading [ " + sFileName + " ] wave file....")

        Dim nPlayWaveLoad As Integer = m_objVaxTeleCOM.PlayWaveLoadFile(sFileName)
        If nPlayWaveLoad = -1 Then

            LogError("")
            LogStatus("Loading [ " + sFileName + " ] wave file fail.")

            LoadWaveFileToSession = False
            Exit Function

        End If

        Dim nSessionPlayWave As Integer = m_objVaxTeleCOM.CreateSession()
        If nSessionPlayWave = -1 Then

            LogError("")
            LogStatus("Loading [ " + sFileName + " ] wave file fail.")

            m_objVaxTeleCOM.PlayWaveUnLoadFile(nPlayWaveLoad)

            LoadWaveFileToSession = False
            Exit Function
        End If

        If m_objVaxTeleCOM.PlayWaveStartToSession(nPlayWaveLoad, nSessionPlayWave, True) = False Then

            LogError("")
            LogStatus("Loading [ " + sFileName + " ] wave file fail.")

            m_objVaxTeleCOM.CloseSession(nSessionPlayWave)
            m_objVaxTeleCOM.PlayWaveUnLoadFile(nPlayWaveLoad)

            LoadWaveFileToSession = False
            Exit Function
        End If

        nPlayWaveId = nPlayWaveLoad
        nSessionId = nSessionPlayWave

        LogStatus("Loading [ " + sFileName + " ] wave file done.")
        LoadWaveFileToSession = True

    End Function

    Public Sub UnLoadWaveFileToSession(ByVal nPlayWaveId As Integer, ByVal nSessionId As Integer)
        m_objVaxTeleCOM.CloseSession(nSessionId)
        m_objVaxTeleCOM.PlayWaveUnLoadFile(nPlayWaveId)
    End Sub

    Public Function AddLine(ByVal nLineId As Integer, ByVal sUserName As String, ByVal sLogin As String, ByVal sLoginPwd As String, ByVal sDisplayName As String, ByVal sDomainRealm As String, ByVal sProxyURI As String, ByVal sOutboundProxy As String) As Boolean
        AddLine = m_objVaxTeleCOM.AddLine(nLineId, sUserName, sLogin, sLoginPwd, sDisplayName, sDomainRealm, sProxyURI, sOutboundProxy)
    End Function

    Public Function RegisterLine(ByVal nLineId As Integer) As Boolean
        RegisterLine = m_objVaxTeleCOM.RegisterLine(nLineId)
    End Function

    Public Function UnRegisterLine(ByVal nLineId As Integer) As Boolean
        UnRegisterLine = m_objVaxTeleCOM.UnRegisterLine(nLineId)
    End Function

    Public Function PlayWaveLoadFile(ByVal sFileName As String) As Integer
        PlayWaveLoadFile = m_objVaxTeleCOM.PlayWaveLoadFile(sFileName)
    End Function

    Public Sub PlayWaveUnLoadFile(ByVal nPlayWaveId As Integer)
        m_objVaxTeleCOM.PlayWaveUnLoadFile(nPlayWaveId)
    End Sub

    Public Function SetVaxTeleTick(ByVal nTickId As Integer, ByVal nElapse As Integer) As Boolean
        SetVaxTeleTick = m_objVaxTeleCOM.SetVaxTeleTick(nTickId, nElapse)
    End Function

    Public Function KillVaxTeleTick(ByVal nTickId As Integer) As Boolean
        KillVaxTeleTick = m_objVaxTeleCOM.KillVaxTeleTick(nTickId)
    End Function

    Public Function AddUser(ByVal sUserLogin As String, ByVal sLoginPwd As String) As Boolean
        AddUser = m_objVaxTeleCOM.AddUser(sUserLogin, sLoginPwd)
    End Function

    Public Sub RemoveUser(ByVal sUserLogin As String)
        m_objVaxTeleCOM.RemoveUser(sUserLogin)
    End Sub

    Public Function GetVaxObjectError() As Integer
        GetVaxObjectError = m_objVaxTeleCOM.GetVaxObjectError()
    End Function

    Public Sub OnTelnetCommand(ByVal sCommand As String, ByRef objParamList As ArrayList, ByRef objTelnetClient As cTelnetClient)

        Dim objTelnetCmd As New cTelnetCmd

        If String.Compare(sCommand, "ShowUsers", True) = 0 Then
            objTelnetCmd.ShowUsers(objTelnetClient)
            Exit Sub
        End If



        If String.Compare(sCommand, "UserInfo", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.UserInfo(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "AddUser", True) = 0 And objParamList.Count = 5 Then

            objTelnetCmd.AddUser(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "RemoveUser", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.RemoveUser(objParamList, objTelnetClient)
            Exit Sub
        End If


        If String.Compare(sCommand, "ShowQueues", True) = 0 Then

            objTelnetCmd.ShowQueues(objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "QueueInfo", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.QueueInfo(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "AddQueue", True) = 0 And objParamList.Count = 6 Then

            objTelnetCmd.AddQueue(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "RemoveQueue", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.RemoveQueue(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "ShowLines", True) = 0 Then

            objTelnetCmd.ShowLines(objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "LineInfo", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.LineInfo(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "AddLine", True) = 0 And objParamList.Count = 12 Then

            objTelnetCmd.AddLine(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "RemoveLine", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.RemoveLine(objParamList, objTelnetClient)
            Exit Sub
        End If

        '///////////////////////////

        If String.Compare(sCommand, "ShowDialPlans", True) = 0 Then

            objTelnetCmd.ShowDialPlans(objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "DialPlanInfo", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.DialPlanInfo(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "AddDialPlan", True) = 0 And objParamList.Count = 3 Then

            objTelnetCmd.AddDialPlan(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "RemoveDialPlan", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.RemoveDialPlan(objParamList, objTelnetClient)
            Exit Sub
        End If

        '///////////////////////////

        If String.Compare(sCommand, "SetStealthMusic", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.SetStealthMusic(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "GetStealthMusic", True) = 0 Then

            objTelnetCmd.GetStealthMusic(objTelnetClient)
            Exit Sub
        End If

        '//////////////////////////

        If String.Compare(sCommand, "SetHoldMusic", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.SetHoldMusic(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "GetHoldMusic", True) = 0 Then

            objTelnetCmd.GetHoldMusic(objTelnetClient)
            Exit Sub
        End If

        '//////////////////////////

        If String.Compare(sCommand, "SetDialToneMusic", True) = 0 And objParamList.Count = 1 Then

            objTelnetCmd.SetDialToneMusic(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "GetDialToneMusic", True) = 0 Then

            objTelnetCmd.GetDialToneMusic(objTelnetClient)
            Exit Sub
        End If

        '//////////////////////////

        If String.Compare(sCommand, "SetCOMM", True) = 0 And objParamList.Count = 3 Then

            objTelnetCmd.SetCOMM(objParamList, objTelnetClient)
            Exit Sub
        End If

        If String.Compare(sCommand, "GetCOMM", True) = 0 Then

            objTelnetCmd.GetCOMM(objTelnetClient)
            Exit Sub
        End If

        '//////////////////////////

        If String.Compare(sCommand, "ServerStart", True) = 0 Then

            objTelnetCmd.StartServer(objTelnetClient)

            StartServerSIP()
            Exit Sub
        End If

        If String.Compare(sCommand, "ServerStop", True) = 0 Then

            objTelnetCmd.StopServer(objTelnetClient)

            StopServerSIP()
            Exit Sub
        End If

        If String.Compare(sCommand, "StatusServerSIP", True) = 0 Then

            Dim objSendParamList As New ArrayList

            If (m_bStatusRun) Then

                objSendParamList.Add("SIP Server is running.")
                objTelnetClient.SendCommand("StatusServerSIP", objSendParamList)

            Else

                objSendParamList.Add("SIP Server is not running.")
                objTelnetClient.SendCommand("StatusServerSIP", objSendParamList)
            End If
        End If

        '////////////////////////

        If String.Compare(sCommand, "GetCallCount", True) = 0 Then

            Dim objSendParamList As New ArrayList

            Dim sCallCount As String = Convert.ToString(m_objCallList.GetCount())
            objSendParamList.Add(sCallCount)

            objTelnetClient.SendCommand("GetCallCount", objSendParamList)
            Exit Sub
        End If

        If String.Compare(sCommand, "GetUserCount", True) = 0 Then

            Dim objSendParamList As New ArrayList

            Dim sUserCount As String = Convert.ToString(m_objUserList.GetCount())
            objSendParamList.Add(sUserCount)

            objTelnetClient.SendCommand("GetUserCount", objSendParamList)
            Exit Sub
        End If

        If String.Compare(sCommand, "GetUsageCPU", True) = 0 Then

            Dim objSendParamList As New ArrayList

            Dim sUsageCPU As String = Convert.ToString(CInt(m_objUsageCPU.NextValue()))
            objSendParamList.Add(sUsageCPU)

            objTelnetClient.SendCommand("GetUsageCPU", objSendParamList)
            Exit Sub
        End If

        If String.Compare(sCommand, "GetAvailMEM", True) = 0 Then

            Dim objSendParamList As New ArrayList()

            Dim objUsageMEM As PerformanceCounter = New PerformanceCounter("Memory", "Available KBytes")
            Dim sAvailMEM As String = Convert.ToString(objUsageMEM.NextValue())

            objSendParamList.Add(sAvailMEM)

            objTelnetClient.SendCommand("GetAvailMEM", objSendParamList)
            Exit Sub
        End If

        '///////////////////////

        objTelnetCmd.InvalidCommand(objTelnetClient)

    End Sub

End Class
