#include "StdAfx.h"
#include "AddLoginForm.h"
#include "Codec.h"
#include "MainForm.h"

using namespace VaxTeleConfigSIP;

bool AddLoginForm::AddLoginChecks()
{
	if(ComboLogin->Text->Length == 0)
	{
		MessageBox::Show("Please enter login.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
        return false;
	}
	
	String^ sDigit = gcnew String("0123456789");

	for(int nCount = 0; nCount < ComboLogin->Text->Length; nCount++)
    {
		if(sDigit->IndexOf(ComboLogin->Text[nCount]) == -1)
		{
			MessageBox::Show("Login should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
			return false;
		}
	}

    if(EditPassword->Text->Length == 0)
	{
		MessageBox::Show("Please enter login password.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return false;
	}

	if(ComboLogin->Text->Length > 20 || EditPassword->Text->Length > 20)
	{
		MessageBox::Show("Login && password size should not be more 20.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return false;
	}

	if(ComboLogin->Text->Substring(0, 2) == "00")
	{
		MessageBox::Show("Please donot enter 00 as login prefix. 00 prefix is use to prepare dial plan.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return false;
	}

	if(ListBoxEnableCodec->Items->Count == 0)
	{
		MessageBox::Show("Please select a codec.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return false;
	}

	return true;
}

void AddLoginForm::BtnAddLogin_Click(System::Object^  sender, System::EventArgs^  e)
{
	if(AddLoginChecks() == false)
	    return;

	if(ComboLogin->FindStringExact(ComboLogin->Text) != -1)
	{
		if(MessageBox::Show("Login already exist, do you want to update it?", "VaxTeleSIP", MessageBoxButtons::YesNo, MessageBoxIcon::Exclamation) == ::DialogResult::No)
			return;
	}

	cCodec^ objCodec = gcnew cCodec();
	String^ sCodec = gcnew String("");

	for(int nCount = 0; nCount < ListBoxEnableCodec->Items->Count; nCount++)
		sCodec += objCodec->GetCodecNo(ListBoxEnableCodec->Items[nCount]->ToString());

	ArrayList^ objParamList = gcnew ArrayList();

	objParamList->Add(ComboLogin->Text);
	objParamList->Add(EditPassword->Text);
	objParamList->Add(sCodec);

	if(CheckBoxStealthListener->Checked)
		objParamList->Add("STEALTH-YES");
	else
		objParamList->Add("STEALTH-NO");

	if(CheckBoxBlockAccount->Checked)
		objParamList->Add("BLOCK-YES");
	else
		objParamList->Add("BLOCK-NO");

	MainForm::m_objMainForm->SendCommand("AddUser", objParamList);
}

void AddLoginForm::UpdateDlgContent(String^ sLogin, String^ sPassword, String^ sCodecList, String^ sStealth, String^ sBlock)
{
	if(ComboLogin->Text != sLogin)
		return;
            
	EditPassword->Text = sPassword;

	ListBoxEnableCodec->Items->Clear();
	ListBoxDisableCodec->Items->Clear();

	cCodec^ objCodec = gcnew cCodec();

	for(int nCount = 0; nCount < TOTAL_SIZE_CODEC; nCount++)
		ListBoxDisableCodec->Items->Add(objCodec->GetCodecName(Convert::ToString(nCount)));

	String^ sCodecName;

	for(int nCount = 0; nCount < sCodecList->Length; nCount++)
	{
		sCodecName = objCodec->GetCodecName(sCodecList[nCount].ToString());

		ListBoxEnableCodec->Items->Add(sCodecName);
		ListBoxDisableCodec->Items->Remove(sCodecName);
	}

	if(sBlock == "BLOCK-YES")
		CheckBoxBlockAccount->Checked = true;
	else
		CheckBoxBlockAccount->Checked = false;

	if(sStealth == "STEALTH-YES")
		CheckBoxStealthListener->Checked = true;
	else
		CheckBoxStealthListener->Checked = false;
}

void AddLoginForm::BtnDisableCodec_Click(System::Object^  sender, System::EventArgs^  e) 
{
	if(ListBoxEnableCodec->SelectedItems->Count == 0) return;

	ListBoxDisableCodec->Items->Add(ListBoxEnableCodec->SelectedItem->ToString());
	ListBoxEnableCodec->Items->Remove(ListBoxEnableCodec->SelectedItem);
}

void AddLoginForm::BtnEnableCodec_Click(System::Object^  sender, System::EventArgs^  e) 
{
	if(ListBoxDisableCodec->SelectedItems->Count == 0) return;

	ListBoxEnableCodec->Items->Add(ListBoxDisableCodec->SelectedItem->ToString());
	ListBoxDisableCodec->Items->Remove(ListBoxDisableCodec->SelectedItem);
}

void AddLoginForm::ClearDlgContent()
{
	ComboLogin->Text = "";
	EditPassword->Text = "";

	ListBoxEnableCodec->Items->Clear();
	ListBoxDisableCodec->Items->Clear();

	CheckBoxBlockAccount->Checked = false;
    CheckBoxStealthListener->Checked = false;

    ListBoxEnableCodec->Items->Add(G711U_TEXT);
    ListBoxEnableCodec->Items->Add(G711A_TEXT);
	ListBoxEnableCodec->Items->Add(ILBC_TEXT);
	ListBoxEnableCodec->Items->Add(G610_TEXT);
	ListBoxEnableCodec->Items->Add(G729_TEXT);
}

void AddLoginForm::AddLoginForm_Load(System::Object^  sender, System::EventArgs^  e)
{
	MainForm::m_objMainForm->SendCommand("ShowUsers");
	ClearDlgContent();
}

void AddLoginForm::ComboLogin_SelectedIndexChanged(System::Object^  sender, System::EventArgs^  e)
{
	ArrayList^ objParamList = gcnew ArrayList();
	objParamList->Add(ComboLogin->SelectedItem->ToString());

	MainForm::m_objMainForm->SendCommand("UserInfo", objParamList);
}

void AddLoginForm::BtnClearLoginDlg_Click(System::Object^  sender, System::EventArgs^  e) 
{
	ClearDlgContent();
}

void AddLoginForm::BtnRemoveLogin_Click(System::Object^  sender, System::EventArgs^  e) 
{
	if(ComboLogin->Text->Length == 0)
	{
		MessageBox::Show("Please enter login.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return;
	}

	ArrayList^ objParamList = gcnew ArrayList();
	objParamList->Add(ComboLogin->Text);

	MainForm::m_objMainForm->SendCommand("RemoveUser", objParamList);
}

void AddLoginForm::OnTelnetCommand(String^ sCommand, ArrayList^ objParamList)
{
	if(String::Compare(sCommand, "ShowUsers", true) == 0)
	{
		for(int nCount = 0; nCount < objParamList->Count; nCount++)
			ComboLogin->Items->Add(objParamList[nCount]);
	    
		if(ComboLogin->Items->Count > 0)
			ComboLogin->SelectedIndex = 0;
	    
		return;
	}

	if(String::Compare(sCommand, "UserInfo", true) == 0 && objParamList->Count == 5)
	{
		String^ sLogin = (String^) objParamList[0];
		String^ sPassword = (String^) objParamList[1];
		String^ sCodecList = (String^) objParamList[2];
		String^ sStealth = (String^) objParamList[3];
		String^ sBlock = (String^) objParamList[4];

		UpdateDlgContent(sLogin, sPassword, sCodecList, sStealth, sBlock);

		return;
	}

	if(String::Compare(sCommand, "RemoveUser", true) == 0 && objParamList->Count == 1)
	{
		int nIndex = ComboLogin->FindStringExact((String^) objParamList[0]);
		if(nIndex == -1) return;
	    
		ComboLogin->Items->RemoveAt(nIndex);

		MessageBox::Show("Login removed successfully!", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		ClearDlgContent();

		return;
	}

	if(String::Compare(sCommand, "AddUser", true) == 0 && objParamList->Count == 1)
	{
		ComboLogin->Items->Add(objParamList[0]);
		MessageBox::Show("Login created successfully", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);

		return;
	}

	if(String::Compare(sCommand, "UpdateUser", true) == 0 && objParamList->Count == 1)
	{
		MessageBox::Show("Login updated successfully", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return;
	}
}

