#include "StdAfx.h"
#include "ServerSIP.h"
#include "QueueList.h"
#include "Registry.h"
#include "CallList.h"
#include "UserList.h"
#include "ServerData.h"

#define DETECT_DIGIT_TIME_START	1
#define DETECT_DIGIT_TIME_UP	2

cQueueData::cQueueData()
{
	m_nQueueDigitDetectPlayWaveId = -1;
	m_unQueueId = 0;
	m_nQueuePlayWaveId = -1;

	m_objAgentList = gcnew ArrayList();
    m_objQueueCallList = gcnew ArrayList();

	m_objDigitDetectList = gcnew Hashtable();
}

cQueueList::cQueueList(void)
{
	m_objQueueListTEXT = gcnew Hashtable();
    m_objQueueListINT  = gcnew Hashtable();

	m_unQueueIdCount = 0;
}

void cQueueList::UnLoadQueue()
{
    DeleteAll();
}

unsigned int cQueueList::ChooseQueueId()
{
    while (true)
    {
        m_unQueueIdCount = m_unQueueIdCount + 1;
        if (FindQueue(m_unQueueIdCount) == nullptr) break;
    }

    return m_unQueueIdCount;
}

void cQueueList::LoadQueue()
{
    cRegistry^ objReg = gcnew cRegistry();
    ArrayList^ objQueueList = gcnew ArrayList();

    if(objReg->EnumQueue(objQueueList) == false)
        return;

    cQueueReg^ objQueueReg = gcnew cQueueReg();
    
    for(int nCount = 0; nCount < objQueueList->Count; nCount++)
    {
        if(objReg->GetQueue((String^) objQueueList[nCount], objQueueReg) == false)
        {
            cServerSIP::m_objServerSIP->LogStatus("Unable to load queue [ " + (String^)objQueueList[nCount] + " ] information from registry.");
            continue;
        }

        Add((String^) objQueueList[nCount], objQueueReg->m_sPlayWave, objQueueReg->m_sPressDigitPlayWave, objQueueReg->m_objQueueOPList, objQueueReg->m_objPressDigitList, objQueueReg->m_objPressDigitUserList);
    }
}

bool cQueueList::AddQueue(String^ sQueueName, String^ sQueuePlayWave, String^ sDigitPressPlayWave, ArrayList^ objAgentList, ArrayList^ objPressDigitList, ArrayList^ objUserDigitList)
{
    cQueueReg^ objQueueReg = gcnew cQueueReg();

    objQueueReg->m_sPlayWave = sQueuePlayWave;
    objQueueReg->m_sPressDigitPlayWave = sDigitPressPlayWave;

    objQueueReg->m_objQueueOPList = objAgentList;
    objQueueReg->m_objPressDigitList = objPressDigitList;
    objQueueReg->m_objPressDigitUserList = objUserDigitList;

    cRegistry^ objReg = gcnew cRegistry();
    if(objReg->SetQueue(sQueueName, objQueueReg) == false)
        return false;

    Add(sQueueName, sQueuePlayWave, sDigitPressPlayWave, objAgentList, objPressDigitList, objUserDigitList);

    return true;
}

bool cQueueList::RemoveQueue(String^ sQueueName)
{
    cRegistry^ objReg = gcnew cRegistry();

    if(objReg->RemoveQueue(sQueueName) == false)
        return false;
    
    Remove(sQueueName);
    return true;
}

bool cQueueList::FindFreeUserId(ArrayList^ objListOP, String^% sFreeUserId)
{
    for(int nCount = 0; nCount < objListOP->Count; nCount++)
    {
        if(cServerSIP::m_objServerSIP->m_objUserList->GetCallCount((String^) objListOP[nCount]) == 0)
        {
            sFreeUserId = (String^) objListOP[nCount];
            return true;
        }
    }

    return false;
}

void cQueueList::PlayQueueWaveToCall(cCallData^ objCallData, int nQueueWaveId, bool bLoopPlay)
{
    if(nQueueWaveId != -1)
        cServerSIP::m_objServerSIP->PlayWaveStartToCall(objCallData, nQueueWaveId, bLoopPlay);
}

bool cQueueList::AddCallToQueue(cCallData^ objCallData)
{
    cQueueData^ objQueueData = FindQueue(objCallData->m_unQueueId);
    if(objQueueData == nullptr) return false;
    
    objQueueData->m_objQueueCallList->Add(objCallData);
    objCallData->m_bDigitDetect = objQueueData->m_bIsDigitDetect;

    UpdateCall(objCallData, objQueueData);
    return true;
}

void cQueueList::UpdateCall(cCallData^ objCallData, cQueueData^ objQueueData)
{
    if(objCallData->m_bDigitDetect)
    {
        cServerSIP::m_objServerSIP->EnableDetectDTMF(objCallData);
        PlayQueueWaveToCall(objCallData, objQueueData->m_nQueueDigitDetectPlayWaveId, false);
    }
    else
    {
        PlayQueueWaveToCall(objCallData, objQueueData->m_nQueuePlayWaveId, true);

        if(objQueueData->m_objQueueCallList->Count == 1)
            DistributeCall(objQueueData);
    }
}

void cQueueList::ProcessQueueTick()
{
    IDictionaryEnumerator^ enumerator  = m_objQueueListINT->GetEnumerator();

    while(enumerator->MoveNext())
    {
        cQueueData^ objQueueData = (cQueueData^) enumerator->Value;
        
		DistributeCall(objQueueData);

		//String^ sFreeAgentUserId = gcnew String("");
		//FindFreeUserId(objQueueData->m_objAgentList, sFreeAgentUserId);
        
    }
}

void cQueueList::DistributeCall(cQueueData^ objQueueData)
{
    cCallData^ objQueueCallData;

    for(int nCount = 0; nCount < objQueueData->m_objQueueCallList->Count; nCount++)
    {
        objQueueCallData = (cCallData^) objQueueData->m_objQueueCallList[nCount];

        if(objQueueCallData->m_bDigitDetect)
        {
            if (DistributeCallToDigitPressUser(objQueueData, objQueueCallData) == false)
                continue;
        }   
        else
        {
            if (DistributeCallToQueueAgent(objQueueData, objQueueCallData) == false)
                return;
        }

        objQueueData->m_objQueueCallList->RemoveAt(nCount);
    }
}

bool cQueueList::DistributeCallToQueueAgent(cQueueData^ objQueueData, cCallData^ objQueueCallData)
{
    String^ sFreeAgentUserId = gcnew String("");

    if(FindFreeUserId(objQueueData->m_objAgentList, sFreeAgentUserId) == false)
        return false;
    
    cCallData^ objDialCallData = cServerSIP::m_objServerSIP->DialCallToUser(objQueueCallData, sFreeAgentUserId);
    if(objDialCallData == nullptr) return false;

    PlayDialToneToCall(objQueueCallData);
    return true;
}

bool cQueueList::DistributeCallToDigitPressUser(cQueueData^ objQueueData, cCallData^ objQueueCallData)
{
    if(objQueueCallData->m_nDigitPRESS == DETECT_DIGIT_TIME_START)
    {
        objQueueCallData->m_nDigitPRESS = DETECT_DIGIT_TIME_UP;
        return false;
    }

    if(objQueueCallData->m_nDigitPRESS == DETECT_DIGIT_TIME_UP)
    {
        if(DialCallToDigitPRESS(objQueueData, objQueueCallData) == true)
            return true;
            
        cServerSIP::m_objServerSIP->DisableDetectDTMF(objQueueCallData);
        UpdateCall(objQueueCallData, objQueueData);
    }

    return false;
}

bool cQueueList::DialCallToDigitPRESS(cQueueData^ objQueueData, cCallData^ objQueueCallData)
{
    if(objQueueCallData->m_sDigitPRESS->Length == 0)
        return false;
        
    String^ sDigitPressUSER = gcnew String("");

    if(FindUserForDigitPRESS(objQueueData, objQueueCallData->m_sDigitPRESS, sDigitPressUSER) == true)
    {
        cCallData^ objDialCallData = cServerSIP::m_objServerSIP->DialCallToUser(objQueueCallData, sDigitPressUSER);

        if(objDialCallData != nullptr)
        {
            cServerSIP::m_objServerSIP->DisableDetectDTMF(objQueueCallData);
            PlayDialToneToCall(objQueueCallData);

            return true;
        }
    }

    return false;
}

bool cQueueList::FindUserForDigitPRESS(cQueueData^ objQueueData, String^ sDigitPRESS, String^ sDigitPressUSER)
{
    sDigitPressUSER = (String^) objQueueData->m_objDigitDetectList[sDigitPRESS];

    if(sDigitPressUSER == nullptr)
        return false;
    
    return true;
}

void cQueueList::OnPressDigitDTMF(cCallData^ objCallData, String^ sDigit)
{
    cServerSIP::m_objServerSIP->PlayWaveStopToCall(objCallData);

    objCallData->m_sDigitPRESS += sDigit;
    objCallData->m_nDigitPRESS = DETECT_DIGIT_TIME_START;
}

void cQueueList::OnPlayWaveDoneToCall(cCallData^ objCallData)
{
    objCallData->m_nDigitPRESS = DETECT_DIGIT_TIME_START;
}

void cQueueList::PlayDialToneToCall(cCallData^ objCallData)
{
    int nDialToneSessionId = cServerSIP::m_objServerSIP->m_objServerData->GetDialToneSessionId();

    if(nDialToneSessionId != -1)
        cServerSIP::m_objServerSIP->AddCallToSession(objCallData, nDialToneSessionId);
}

void cQueueList::RemoveQueueCall(cCallData^ objCallData)
{
    cQueueData^ objQueueData = FindQueue(objCallData->m_unQueueId);

    if(objQueueData == nullptr)
        return;
    
    int nIndex = objQueueData->m_objQueueCallList->IndexOf(objCallData);
    
    if(nIndex != -1)
        objQueueData->m_objQueueCallList->RemoveAt(nIndex);
}

cQueueData^ cQueueList::FindQueue(String^ sQueueName)
{
    return (cQueueData^) m_objQueueListTEXT[sQueueName->ToLower()];
}

cQueueData^ cQueueList::FindQueue(unsigned int unQueueId)
{
    return (cQueueData^) m_objQueueListINT[unQueueId];
}

void cQueueList::DeleteAll()
{
    cQueueData^ objQueueData;
    IDictionaryEnumerator^ enumerator = m_objQueueListINT->GetEnumerator();

    while(enumerator->MoveNext())
    {
        objQueueData = (cQueueData^) enumerator->Value;
        
        cServerSIP::m_objServerSIP->PlayWaveUnLoadFile(objQueueData->m_nQueuePlayWaveId);
        cServerSIP::m_objServerSIP->PlayWaveUnLoadFile(objQueueData->m_nQueueDigitDetectPlayWaveId);
    }

    m_objQueueListTEXT->Clear();
    m_objQueueListINT->Clear();
}

void cQueueList::Remove(String^ sQueueName)
{
    cQueueData^ objQueueData = FindQueue(sQueueName);
    
    if(objQueueData == nullptr)
        return;
            
    for(int nCount = 0; nCount < objQueueData->m_objQueueCallList->Count; nCount++)
    {
        cCallData^ objQueueCallData = (cCallData^) objQueueData->m_objQueueCallList[nCount];
        cServerSIP::m_objServerSIP->CloseCall(objQueueCallData);
    }

    m_objQueueListINT->Remove(objQueueData->m_unQueueId);
    m_objQueueListTEXT->Remove(sQueueName->ToLower());
}

void cQueueList::Add(String^ sQueueName, String^ sQueuePlayWave, String^ sDigitPressPlayWave, ArrayList^ objAgentList, ArrayList^ objPressDigitList, ArrayList^ objUserDigitList)
{
    Remove(sQueueName);

    int nPlayWaveId = -1;
    int nDigitPressPlayWaveId = -1;

    if (sQueuePlayWave->Length != 0)
    {
        cServerSIP::m_objServerSIP->LogStatus("Loading [ " + sQueuePlayWave + " ] wave file....");
        nPlayWaveId = cServerSIP::m_objServerSIP->PlayWaveLoadFile(sQueuePlayWave);

        if (nPlayWaveId == -1)
        {
            cServerSIP::m_objServerSIP->LogError("");
            cServerSIP::m_objServerSIP->LogStatus("Loading [ " + sQueuePlayWave + " ] wave file fail.");
        }
        else
        {
            cServerSIP::m_objServerSIP->LogStatus("Loading [ " + sQueuePlayWave + " ] wave file done.");
        }

    }

    if (sDigitPressPlayWave->Length != 0)
    {
        cServerSIP::m_objServerSIP->LogStatus("Loading [ " + sDigitPressPlayWave + " ] wave file....");
        nDigitPressPlayWaveId = cServerSIP::m_objServerSIP->PlayWaveLoadFile(sDigitPressPlayWave);

        if (nDigitPressPlayWaveId == -1)
        {
            cServerSIP::m_objServerSIP->LogError("");
            cServerSIP::m_objServerSIP->LogStatus("Loading [ " + sQueuePlayWave + " ] wave file fail.");
        }
        else
        {
            cServerSIP::m_objServerSIP->LogStatus("Loading [ " + sQueuePlayWave + " ] wave file done.");
        }
    }

    cQueueData^ objQueueData = gcnew cQueueData();

    if (objPressDigitList->Count == 0)
        objQueueData->m_bIsDigitDetect = false;
    else
        objQueueData->m_bIsDigitDetect = true;

    int nCount;

    for (nCount = 0; nCount < objAgentList->Count; nCount++)
        objQueueData->m_objAgentList->Add(objAgentList[nCount]);

    for (nCount = 0; nCount < objPressDigitList->Count; nCount++)
        objQueueData->m_objDigitDetectList->Add(objPressDigitList[nCount], objUserDigitList[nCount]);

    objQueueData->m_unQueueId = ChooseQueueId();
    objQueueData->m_nQueueDigitDetectPlayWaveId = nDigitPressPlayWaveId;
    objQueueData->m_nQueuePlayWaveId = nPlayWaveId;

    m_objQueueListINT->Add(objQueueData->m_unQueueId, objQueueData);
    m_objQueueListTEXT->Add(sQueueName->ToLower(), objQueueData);

}