#include "StdAfx.h"
#include "VaxTeleCOM.h"
#include "TelnetServer.h"
#include "ServerSIP.h"
#include "CallList.h"
#include "LineList.h"
#include "DialPlanList.h"
#include "QueueList.h"
#include "UserList.h"
#include "StealthList.h"
#include "ServerData.h"
#include "ErrorLog.h"

#include "Registry.h"
#include "TelnetCmd.h"
#include "TelnetClient.h"

#include "ClientRegisterEvent.h"
#include "CallEvent.h"
#include "ResponseEvent.h"
#include "HoldCallEvent.h"
#include "TransferCallEvent.h"
#include "LineRegisterEvent.h"

cServerSIP::cServerSIP(void)
{
	m_objUsageCPU = gcnew PerformanceCounter("Processor", "% Processor Time", "_Total");
	m_objVaxTeleCOM = gcnew cVaxTeleCOM();

	m_objServerSIP = this;
	m_bStatusRun = false;
	
	m_objCallList = gcnew cCallList();
    m_objLineList = gcnew cLineList();
    m_objDialPlanList = gcnew cDialPlanList();
    m_objQueueList = gcnew cQueueList();
    m_objUserList = gcnew cUserList();
    m_objStealthList = gcnew cStealthList();

    m_objServerData = gcnew cServerData();
    m_objErrorLog = gcnew cErrorLog();

    m_objPlayWaveList = gcnew Hashtable();

	m_objClientRegEvent = gcnew cClientRegisterEvent();
    m_objCallEvent = gcnew cCallEvent();
    m_objResponseEvent = gcnew cResponseEvent();
    m_objHoldCallEvent = gcnew cHoldCallEvent();
    m_objTransferCallEvent = gcnew cTransferCallEvent();
    m_objLineRegisterEvent = gcnew cLineRegisterEvent();
}

bool cServerSIP::StartServerSIP()
{
	if(m_bStatusRun == false)
	{
		m_bStatusRun = InitVaxTeleSIP();
	}

	ArrayList^ objParamList = gcnew ArrayList();

	if(m_bStatusRun == true)
	{
		objParamList->Add("SIP Server is running.");
		cTelnetServer::m_objTelnetServer->SendCommandToAll("StatusServerSIP", objParamList);
	}
	else
	{
		objParamList->Add("Unable to start SIP Server (see error log).");
		cTelnetServer::m_objTelnetServer->SendCommandToAll("StatusServerSIP", objParamList);
	}

	return m_bStatusRun;
}

void cServerSIP::StopServerSIP()
 {
	if(m_bStatusRun == true)
	{
		UnInitVaxTeleSIP();
		m_bStatusRun = false;
	}
     
	ArrayList^ objParamList = gcnew ArrayList();
	objParamList->Add("SIP Server is not running.");

	cTelnetServer::m_objTelnetServer->SendCommandToAll("StatusServerSIP", objParamList);
 }

 void cServerSIP::UnInitVaxTeleSIP()
 {
     LogStatus(" ");

     m_objCallList->DeleteAll();
     m_objServerData->UnLoadServer();

     LogStatus("Uninitializing VaxTele SIP Server.");
     m_objVaxTeleCOM->UnInitialize();
     LogStatus("VaxTele SIP Server uninitialized successfully.");

     m_objUserList->DeleteAll();
     LogStatus("VaxTele SIP Server stopped successfully.");
 }

 bool cServerSIP::InitVaxTeleSIP()
 {
     LogStatus(" ");

     cRegistry^ objReg = gcnew cRegistry();
     
     if(objReg->GetStartServer() == false)
         return false;
     
     String^ sListenIP = gcnew String("");
     String^ sDomainRealm = gcnew String("");
     String^ sListenPort = gcnew String("");

     objReg->GetComm(sListenIP, sListenPort, sDomainRealm);

     m_objClientRegEvent->SetDomainRealm(sDomainRealm);
     m_objVaxTeleCOM->SetLicenseKey("TRIAL-LICENSE-KEY");

     LogStatus("Initializing VaxTele SIP Server.");

     if(m_objVaxTeleCOM->Initialize(sDomainRealm, sListenIP, Convert::ToInt32(sListenPort), sListenIP, -1) == false)
     {
         LogStatus("Unable to initialize VaxTele SIP Server (see error log).");
         LogError("");

         return false;
     }

     LogStatus("VaxTele SIP Server initialized successfully.");
     m_objServerData->LoadServer();
     LogStatus("VaxTele SIP Server started successfully.");

     return true;
 }

 void cServerSIP::BindTwoCall(cCallData^ objCallDataA, cCallData^ objCallDataB)
 {
     objCallDataA->m_objBindCallData = objCallDataB;
     objCallDataB->m_objBindCallData = objCallDataA;
 }

 bool cServerSIP::SendResponse(cCallData^ objCallData, int nStatusCode, String^ sReasonPhrase)
 {
     if(objCallData->m_bConnected == true)
         return false;

     return m_objVaxTeleCOM->SendResponse(objCallData->m_nCallId, nStatusCode, sReasonPhrase, "");
 }

 void cServerSIP::CloseCall(cCallData^ objCallData)
 {
     switch(objCallData->m_nCallType)
    {
	    case CALL_TYPE_QUEUE:
		    m_objQueueList->RemoveQueueCall(objCallData);
		    break;
	
	    case CALL_TYPE_STEALTH:
		    m_objStealthList->RemoveStealthCall(objCallData);
		    break;
    }

    PlayWaveStopToCall(objCallData);
    m_objStealthList->FreeCaptureCall(objCallData);
	
    CloseSession(objCallData);
    m_objVaxTeleCOM->CloseCall(objCallData->m_nCallId);
		
    cUserData^ objUserData = m_objUserList->FindUser(objCallData->m_unUserId);
    if(objUserData != nullptr) objUserData->m_nCallCount--;

    m_objCallList->RemoveCall(objCallData);
 }

 cCallData^ cServerSIP::DialCallToLine(cCallData^ objFromCallData, String^ sDialNo, String^ sLineName)
 {
     cLineData^ objLineData = m_objLineList->FindLine(sLineName);

     if(objLineData == nullptr)
     {
         SendResponse(objFromCallData, 404, "User Not Found");
         CloseCall(objFromCallData);

         return nullptr;
     }

     int nDialCall = m_objVaxTeleCOM->DialCallToLine(objLineData->m_nLineId, sDialNo, objLineData->m_sCodecList, 120);
     
     if(nDialCall == -1)
     {
         SendResponse(objFromCallData, 404, "User Not Found");
         CloseCall(objFromCallData);

         return nullptr;
     }

     cCallData^ objDialCallData = m_objCallList->AddCall(nDialCall);
     BindTwoCall(objFromCallData,objDialCallData);

     return objDialCallData;
 }

 cCallData^ cServerSIP::DialCallToUser(cCallData^ objFromCallData, String^ sToUserId)
 {
     cUserData^ objUserData = m_objUserList->FindUser(sToUserId);

     if(objUserData == nullptr)
     {
         SendResponse(objFromCallData, 404, "User Not Found");
         CloseCall(objFromCallData);

         return nullptr;
     }

     int nDialCall = m_objVaxTeleCOM->DialCallToUser(objFromCallData->m_sCallerId, sToUserId, objUserData->m_sCodecList, 120);
     
     if(nDialCall == -1)
     {
         SendResponse(objFromCallData, 404, "User Not Found");
         CloseCall(objFromCallData);

         return nullptr;
     }

     objUserData->m_nCallCount++;

     cCallData^ objDialCallData = m_objCallList->AddCall(nDialCall);

     objDialCallData->m_sUserId = sToUserId;
     objDialCallData->m_unUserId = objUserData->m_unUserId;

     BindTwoCall(objFromCallData,objDialCallData);

     return objDialCallData;
 }

 bool cServerSIP::AcceptCall(cCallData^ objCallData)
 {
     bool bResult = m_objVaxTeleCOM->AcceptCall(objCallData->m_nCallId, objCallData->m_sCodec, 120);

     if (bResult == false)
     {
         LogError("");
         LogStatus("cServerSIP.AcceptCall() method fails.");
     }

     return bResult;
 }

 void cServerSIP::CreateSession(cCallData^ objCallData)
 {
     CloseSession(objCallData);

     objCallData->m_nSessionId = m_objVaxTeleCOM->CreateSession();
     m_objVaxTeleCOM->AddCallToSession(objCallData->m_nCallId, objCallData->m_nSessionId);
 }

 void cServerSIP::AddCallToSession(cCallData^ objCallData, int nSessionId)
 {
     m_objVaxTeleCOM->AddCallToSession(objCallData->m_nCallId, nSessionId);
 }

 void cServerSIP::CloseSession(cCallData^ objCallData)
 {
     if(objCallData->m_nSessionId == -1)
         return;
     
     PlayWaveStopToCall(objCallData);

     m_objVaxTeleCOM->CloseSession(objCallData->m_nSessionId);
     objCallData->m_nSessionId = -1;
 }

 void cServerSIP::TransferCallAccept(cCallData^ objCallData)
 {
     m_objVaxTeleCOM->TransferCallAccept(objCallData->m_nCallId);
 }

 void cServerSIP::DisableDetectDTMF(cCallData^ objCallData)
 {
     objCallData->m_bDigitDetect = false;
     objCallData->m_nDigitPRESS = 0;
     objCallData->m_sDigitPRESS = "";

     m_objVaxTeleCOM->DetectDTMF(0, objCallData->m_nSessionId, false); // Disable RFC2833 DTMF detection
 }

 void cServerSIP::EnableDetectDTMF(cCallData^ objCallData)
 {
     objCallData->m_bDigitDetect = true;
     objCallData->m_nDigitPRESS = 0;
     objCallData->m_sDigitPRESS = "";

     if (!m_objVaxTeleCOM->DetectDTMF(0, objCallData->m_nSessionId, true)) //Enable RFC2833 DTMF detection
         LogError("");
 }

 bool cServerSIP::RejectClientRegister(int nRegId)
 {
     return m_objVaxTeleCOM->RejectClientRegister(nRegId);
 }

 bool cServerSIP::AuthClientRegister(int nRegId)
 {
     return m_objVaxTeleCOM->AuthClientRegister(nRegId);
 }

 void cServerSIP::OnClientUnRegister(String^ sUserLogin)
 {
     m_objClientRegEvent->OnClientUnRegister(sUserLogin);
 }

 void cServerSIP::OnClientRegister(String^ sUserLogin, String^ sDomain, int nRegId)
 {
     m_objClientRegEvent->OnClientRegister(sUserLogin, sDomain, nRegId);
 }

 void cServerSIP::OnClientRegisterSuccess(String^ sUserLogin)
 {
     m_objClientRegEvent->OnClientRegisterSuccess(sUserLogin);
 }

 void cServerSIP::OnClientRegisterFail(String^ sUserLogin)
 {
     m_objClientRegEvent->OnClientRegisterFail(sUserLogin);
 }

 void cServerSIP::OnIncomingCallAnonymous(int nCallId, String^ sCallFrom, String^ sCallTo, String^ sFromURI, String^ sToURI, String^ sFromIP, int nFromPort)
 {
     m_objVaxTeleCOM->SendResponse(nCallId, 404, "Anonymous not allowed.", "");
 }

 void cServerSIP::OnIncomingCallFromLine(int nCallId, int nLineId, String^ sCallFrom, String^ sCallTo, String^ sFromURI, String^ sToURI, String^ sFromIP, int nFromPort)
 {
     cLineData^ objLineData = m_objLineList->FindLine(nLineId);

     if(objLineData == nullptr)
     {
         m_objVaxTeleCOM->CloseCall(nCallId);
         return;
     }

     cCallData^ objCallData = m_objCallList->AddCall(nCallId);

     objCallData->m_sCodec = objLineData->m_sCodecList;
     objCallData->m_sCallerId = sCallFrom;

     CreateSession(objCallData);

     m_objCallEvent->OnIncomingCallFromLine(objCallData, objLineData, sCallTo);
 }

 void cServerSIP::OnIncomingCallFromUser(int nCallId, String^ sUserName, String^ sCallFrom, String^ sCallTo, String^ sFromURI, String^ sToURI, String^ sFromIP, int nFromPort)
 {
     cUserData^ objUserData = m_objUserList->FindUser(sUserName);

     if(objUserData == nullptr)
         return;
     
     cCallData^ objCallData = m_objCallList->AddCall(nCallId);

     objCallData->m_sCodec = objUserData->m_sCodecList;
     objCallData->m_unUserId = objUserData->m_unUserId;
     objCallData->m_sUserId = sUserName;
     objCallData->m_sCallerId = sCallFrom;

     CreateSession(objCallData);

     m_objVaxTeleCOM->EnableDialTone(nCallId, m_objServerData->GetDialToneSessionId());
     m_objCallEvent->OnIncomingCallFromUser(objCallData, objUserData, sCallTo);
 }

 void cServerSIP::StartVoiceStream(cCallData^ objCallDataA, cCallData^ objCallDataB)
 {
     CloseSession(objCallDataA);
     CloseSession(objCallDataB);

     int nSessionId = m_objVaxTeleCOM->CreateSession();

     objCallDataA->m_nSessionId = nSessionId;
     objCallDataB->m_nSessionId = nSessionId;

     AddCallToSession(objCallDataA, nSessionId);
     AddCallToSession(objCallDataB, nSessionId);

 }

 void cServerSIP::OnAcceptIncomingCallConnected(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objCallEvent->OnAcceptIncomingCallConnected(objCallData);
 }

 void cServerSIP::OnDialCallConnected(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objCallEvent->OnDialCallConnected(objCallData);
 }

 void cServerSIP::OnDialCallTimeOut(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objCallEvent->OnDialCallTimeOut(objCallData);
 }

 void cServerSIP::OnAcceptCallTimeOut(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objCallEvent->OnAcceptCallTimeOut(objCallData);
 }

 void cServerSIP::OnDisconnectCall(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objCallEvent->OnDisconnectCall(objCallData);
 }

 void cServerSIP::OnIncomingCallCancel(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objCallEvent->OnIncomingCallCancel(objCallData);
 }

 void cServerSIP::OnSessionLost(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;

     m_objCallEvent->OnSessionLost(objCallData);
 }

 void cServerSIP::OnProvisionalResponse(int nCallId, int nStatusCode, String^ sReasonPhrase)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objResponseEvent->OnProvisionalResponse(objCallData, nStatusCode, sReasonPhrase);
 }

 void cServerSIP::OnFailureResponse(int nCallId, int nStatusCode, String^ sReasonPhrase)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objResponseEvent->OnFailureResponse(objCallData, nStatusCode, sReasonPhrase);
 }

 void cServerSIP::OnRedirectionResponse(int nCallId, int nStatusCode, String^ sReasonPhrase, String^ sContact)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objResponseEvent->OnRedirectionResponse(objCallData, nStatusCode, sReasonPhrase, sContact);
 }

 void cServerSIP::OnDigitDetectDTMF(int nSessionId, int nCallId, int nDigit)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     String^ sDigit = "0123456789*#ABCDEF";

     m_objQueueList->OnPressDigitDTMF(objCallData, Convert::ToString(sDigit[nDigit]));
 }

 void cServerSIP::OnHoldCall(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objHoldCallEvent->OnHoldCall(objCallData);

 }

 void cServerSIP::OffHoldCall(int nCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     m_objHoldCallEvent->OffHoldCall(objCallData);
 }

 void cServerSIP::OnTransferCallBlind(int nCallId, String^ sTransferee, String^ sTransferTo, String^ sTransferToURI)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;

     m_objTransferCallEvent->OnTransferCallBlind(objCallData, sTransferee, sTransferTo, sTransferToURI);
 }

 void cServerSIP::OnTransferCallAttend(int nCallId, int nTransferToCallId)
 {
     cCallData^ objCallData = m_objCallList->FindCall(nCallId);
     
     if(objCallData == nullptr)
         return;
     
     cCallData^ objTransferToCallData = m_objCallList->FindCall(nTransferToCallId);
     
     if(objTransferToCallData == nullptr)
         return;
     
     m_objTransferCallEvent->OnTransferCallAttend(objCallData, objTransferToCallData);
 }

 bool cServerSIP::PlayWaveStopToCall(cCallData^ objCallData)
 {
     if(objCallData->m_nPlayWaveType == PLAY_WAVE_NONE_TYPE)
         return true;
     
     if(objCallData->m_nPlayWaveType == PLAY_WAVE_DONE_TYPE)
         m_objPlayWaveList->Remove(objCallData->m_nSessionId);
     
     objCallData->m_nPlayWaveType = PLAY_WAVE_NONE_TYPE;

     return m_objVaxTeleCOM->PlayWaveStopToSession(objCallData->m_nSessionId);
 }

 bool cServerSIP::PlayWaveStartToCall(cCallData^ objCallData, int nPlayWaveId, bool bLoopPlay)
 {
     if(bLoopPlay == false)
     {
         objCallData->m_nPlayWaveType = PLAY_WAVE_DONE_TYPE;
         m_objPlayWaveList->Add(objCallData->m_nSessionId, objCallData);
     }
     else
     {
         objCallData->m_nPlayWaveType = PLAY_WAVE_LOOP_TYPE;
     }

     return m_objVaxTeleCOM->PlayWaveStartToSession(nPlayWaveId, objCallData->m_nSessionId, bLoopPlay);
 }

 void cServerSIP::OnPlayWaveToSessionDone(int nPlayWaveId, int nSessionId)
 {
     cCallData^ objCallData = (cCallData^) m_objPlayWaveList[nSessionId];

     if(objCallData == nullptr)
         return;
     
     m_objPlayWaveList->Remove(nSessionId);
     objCallData->m_nPlayWaveType = PLAY_WAVE_NONE_TYPE;

     m_objQueueList->OnPlayWaveDoneToCall(objCallData);
 }

 void cServerSIP::OnLineRegisterTrying(int nLineId)
 {
     m_objLineRegisterEvent->OnLineRegisterTrying(nLineId);
 }

 void cServerSIP::OnLineRegisterFail(int nLineId, int nStatusCode, String^ sReasonPhrase)
 {
     m_objLineRegisterEvent->OnLineRegisterFail(nLineId, nStatusCode, sReasonPhrase);
 }

 void cServerSIP::OnLineRegisterSuccess(int nLineId)
 {
     m_objLineRegisterEvent->OnLineRegisterSuccess(nLineId);
 }

 void cServerSIP::OnLineUnRegisterTrying(int nLineId)
 {
     m_objLineRegisterEvent->OnLineUnRegisterTrying(nLineId);
 }

 void cServerSIP::OnLineUnRegisterFail(int nLineId, int nStatusCode, String^ sReasonPhrase)
 {
     m_objLineRegisterEvent->OnLineUnRegisterFail(nLineId, nStatusCode, sReasonPhrase);
 }

 void cServerSIP::OnLineUnRegisterSuccess(int nLineId)
 {
     m_objLineRegisterEvent->OnLineUnRegisterSuccess(nLineId);
 }

 void cServerSIP::OnVaxTeleTick(int nTickId)
 {
     if(nTickId == QUEUE_TICK_ID)
         m_objQueueList->ProcessQueueTick();
 }

 void cServerSIP::LogError(String^ sError)
 {
     m_objErrorLog->LogError(sError);
 }

 void cServerSIP::LogStatus(String^ sStatus)
 {
     ArrayList^ objParamList = gcnew ArrayList();
     objParamList->Add(sStatus);

     cTelnetServer::m_objTelnetServer->SendCommandToAll("StatusLog", objParamList);
 }

 bool cServerSIP::LoadWaveFileToSession(String^ sFileName, int% nPlayWaveId, int% nSessionId)
 {
     LogStatus("Loading [ " + sFileName + " ] wave file....");

     int nPlayWaveLoad = m_objVaxTeleCOM->PlayWaveLoadFile(sFileName);
     
     if(nPlayWaveLoad == -1)
     {
         LogError("");
         LogStatus("Loading [ " + sFileName + " ] wave file fail.");

         return false;
     }

     int nSessionPlayWave = m_objVaxTeleCOM->CreateSession();
     
     if(nSessionPlayWave == -1)
     {
         LogError("");
         LogStatus("Loading [ " + sFileName + " ] wave file fail.");

         m_objVaxTeleCOM->PlayWaveUnLoadFile(nPlayWaveLoad);

         return false;
     }

     if(m_objVaxTeleCOM->PlayWaveStartToSession(nPlayWaveLoad, nSessionPlayWave, true) == false)
     {
         LogError("");
         LogStatus("Loading [ " + sFileName + " ] wave file fail.");

         m_objVaxTeleCOM->CloseSession(nSessionPlayWave);
         m_objVaxTeleCOM->PlayWaveUnLoadFile(nPlayWaveLoad);

         return false;
     }

     nPlayWaveId = nPlayWaveLoad;
     nSessionId = nSessionPlayWave;

     LogStatus("Loading [ " + sFileName + " ] wave file done.");
     return true;

 }

 void cServerSIP::UnLoadWaveFileToSession(int nPlayWaveId, int nSessionId)
 {
     m_objVaxTeleCOM->CloseSession(nSessionId);
     m_objVaxTeleCOM->PlayWaveUnLoadFile(nPlayWaveId);
 }

 bool cServerSIP::AddLine(int nLineId, String^ sUserName, String^ sLogin, String^ sLoginPwd, String^ sDisplayName, String^ sDomainRealm, String^ sProxyURI, String^ sOutboundProxy)
 {
     return m_objVaxTeleCOM->AddLine(nLineId, sUserName, sLogin, sLoginPwd, sDisplayName, sDomainRealm, sProxyURI, sOutboundProxy);
 }

 bool cServerSIP::RegisterLine(int nLineId)
 {
     return m_objVaxTeleCOM->RegisterLine(nLineId);
 }

 bool cServerSIP::UnRegisterLine(int nLineId)
 {
     return m_objVaxTeleCOM->UnRegisterLine(nLineId);
 }

 int cServerSIP::PlayWaveLoadFile(String^ sFileName)
 {
     return m_objVaxTeleCOM->PlayWaveLoadFile(sFileName);
 }

 void cServerSIP::PlayWaveUnLoadFile(int nPlayWaveId)
 {
     m_objVaxTeleCOM->PlayWaveUnLoadFile(nPlayWaveId);
 }

 bool cServerSIP::SetVaxTeleTick(int nTickId, int nElapse)
 {
     return m_objVaxTeleCOM->SetVaxTeleTick(nTickId, nElapse);
 }

 bool cServerSIP::KillVaxTeleTick(int nTickId)
 {
     return m_objVaxTeleCOM->KillVaxTeleTick(nTickId);
 }

 bool cServerSIP::AddUser(String^ sUserLogin, String^ sLoginPwd)
 {
     return m_objVaxTeleCOM->AddUser(sUserLogin, sLoginPwd);
 }

 void cServerSIP::RemoveUser(String^ sUserLogin)
 {
     m_objVaxTeleCOM->RemoveUser(sUserLogin);
 }

 int cServerSIP::GetVaxObjectError()
 {
     return m_objVaxTeleCOM->GetVaxObjectError();
 }

 void cServerSIP::OnTelnetCommand(String^ sCommand, ArrayList^ objParamList, cTelnetClient^ objTelnetClient)
 {
     cTelnetCmd^ objTelnetCmd = gcnew cTelnetCmd();

     if(String::Compare(sCommand, "ShowUsers", true) == 0)
     {
         objTelnetCmd->ShowUsers(objTelnetClient);
         return;
     }
 

     if(String::Compare(sCommand, "UserInfo", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->UserInfo(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "AddUser", true) == 0 && objParamList->Count == 5)
     {
         objTelnetCmd->AddUser(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "RemoveUser", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->RemoveUser(objParamList, objTelnetClient);
         return;
     }
 
     if(String::Compare(sCommand, "ShowQueues", true) == 0)
     {
         objTelnetCmd->ShowQueues(objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "QueueInfo", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->QueueInfo(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "AddQueue", true) == 0 && objParamList->Count == 6)
     {
         objTelnetCmd->AddQueue(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "RemoveQueue", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->RemoveQueue(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "ShowLines", true) == 0)
     {
         objTelnetCmd->ShowLines(objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "LineInfo", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->LineInfo(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "AddLine", true) == 0 && objParamList->Count == 12)
     {
         objTelnetCmd->AddLine(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "RemoveLine", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->RemoveLine(objParamList, objTelnetClient);
         return;
     }

     ///////////////////////////

     if(String::Compare(sCommand, "ShowDialPlans", true) == 0)
     {
         objTelnetCmd->ShowDialPlans(objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "DialPlanInfo", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->DialPlanInfo(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "AddDialPlan", true) == 0 && objParamList->Count == 3)
     {
         objTelnetCmd->AddDialPlan(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "RemoveDialPlan", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->RemoveDialPlan(objParamList, objTelnetClient);
         return;
     }

     ///////////////////////////

     if(String::Compare(sCommand, "SetStealthMusic", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->SetStealthMusic(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "GetStealthMusic", true) == 0)
     {
         objTelnetCmd->GetStealthMusic(objTelnetClient);
         return;
     }

     //////////////////////////

     if(String::Compare(sCommand, "SetHoldMusic", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->SetHoldMusic(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "GetHoldMusic", true) == 0)
     {
         objTelnetCmd->GetHoldMusic(objTelnetClient);
         return;
     }

     //////////////////////////

     if(String::Compare(sCommand, "SetDialToneMusic", true) == 0 && objParamList->Count == 1)
     {
         objTelnetCmd->SetDialToneMusic(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "GetDialToneMusic", true) == 0)
     {
         objTelnetCmd->GetDialToneMusic(objTelnetClient);
         return;
     }

     //////////////////////////

     if(String::Compare(sCommand, "SetCOMM", true) == 0 && objParamList->Count == 3)
     {
         objTelnetCmd->SetCOMM(objParamList, objTelnetClient);
         return;
     }

     if(String::Compare(sCommand, "GetCOMM", true) == 0)
     {
         objTelnetCmd->GetCOMM(objTelnetClient);
         return;
     }

     //////////////////////////

     if(String::Compare(sCommand, "ServerStart", true) == 0)
     {
         objTelnetCmd->StartServer(objTelnetClient);

         StartServerSIP();
         return;
     }

     if(String::Compare(sCommand, "ServerStop", true) == 0)
     {
         objTelnetCmd->StopServer(objTelnetClient);

         StopServerSIP();
         return;
     }

     if(String::Compare(sCommand, "StatusServerSIP", true) == 0)
     {
         ArrayList^ objSendParamList = gcnew ArrayList();

         if(m_bStatusRun)
         {
             objSendParamList->Add("SIP Server is running.");
             objTelnetClient->SendCommand("StatusServerSIP", objSendParamList);
         }
         else
         {
             objSendParamList->Add("SIP Server is not running.");
             objTelnetClient->SendCommand("StatusServerSIP", objSendParamList);
         }
     }

     ////////////////////////

     if(String::Compare(sCommand, "GetCallCount", true) == 0)
     {
         ArrayList^ objSendParamList = gcnew ArrayList();

         String^ sCallCount = Convert::ToString(m_objCallList->GetCount());
         objSendParamList->Add(sCallCount);

         objTelnetClient->SendCommand("GetCallCount", objSendParamList);
         return;
     }

     if(String::Compare(sCommand, "GetUserCount", true) == 0)
     {
         ArrayList^ objSendParamList = gcnew ArrayList();

         String^ sUserCount = Convert::ToString(m_objUserList->GetCount());
         objSendParamList->Add(sUserCount);

         objTelnetClient->SendCommand("GetUserCount", objSendParamList);
         return;
     }

     if(String::Compare(sCommand, "GetUsageCPU", true) == 0)
     {
         ArrayList^ objSendParamList = gcnew ArrayList();

         String^ sUsageCPU = Convert::ToString((int) m_objUsageCPU->NextValue());
         objSendParamList->Add(sUsageCPU);

         objTelnetClient->SendCommand("GetUsageCPU", objSendParamList);
         return;
     }

     if (String::Compare(sCommand, "GetAvailMEM", true) == 0)
     {
         ArrayList^ objSendParamList = gcnew ArrayList();
         
         PerformanceCounter^ objUsageMEM = gcnew PerformanceCounter("Memory", "Available KBytes"); 
         String^ sAvailMEM = Convert::ToString(objUsageMEM->NextValue());

         objSendParamList->Add(sAvailMEM);

         objTelnetClient->SendCommand("GetAvailMEM", objSendParamList);
         return;
     }

     ////////////////////////

     objTelnetCmd->InvalidCommand(objTelnetClient);

 }
