#include "StdAfx.h"
#include "TelnetRequest.h"
#include "TelnetClient.h"
#include "TelnetServer.h"
#include "ServerSIP.h"

#define RECV_BUFFER_SIZE	512

cTelnetClient::cTelnetClient(void)
{
	m_objSocket = nullptr;
	m_objTelnetRequest = gcnew cTelnetRequest();
	m_sLoginTelnet = gcnew String("");

	m_sAsyncBuffer = gcnew array<System::Byte>(RECV_BUFFER_SIZE);

	m_nClientListIndex = -1;
	m_nAuthStep = AUTH_LOGIN;
}

void cTelnetClient::OnReceive(IAsyncResult^ ar)
{        
    cTelnetClient^ objTelnetClient = dynamic_cast<cTelnetClient^>(ar->AsyncState);
	
	Monitor::Enter(objTelnetClient);
    
	if (objTelnetClient->m_objSocket == nullptr)
	{
		Monitor::Exit(objTelnetClient);	
		return;
	}

	try
	{
		int nRead = objTelnetClient->m_objSocket->EndReceive(ar);

		if (nRead == 0)
		{
			objTelnetClient->OnDisconnect();
			Monitor::Exit(objTelnetClient);
			return;
		}

		if (objTelnetClient->ReceiveData(Encoding::ASCII->GetString(m_sAsyncBuffer, 0, nRead)) == false)
		{
			Monitor::Exit(objTelnetClient);
			return;
		}

		objTelnetClient->m_objSocket->BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags::None, gcnew AsyncCallback(OnReceive), objTelnetClient);
	}
	catch (SocketException^ e)
	{
		objTelnetClient->OnDisconnect();
	}

	Monitor::Exit(objTelnetClient);
}

void cTelnetClient::OnDisconnect()
{
	CloseSocket();
	cTelnetServer::m_objTelnetServer->OnClientDisconnect(this);
}

void cTelnetClient::OnConnected()
{
	SendText("Welcome to VaxTele!!\r\n\r\n\r\nLogin:");
	m_objSocket->BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags::None, gcnew AsyncCallback(OnReceive), this);
}

bool cTelnetClient::OnRequest(String^ sCommand, ArrayList^ objParamList)
{
	Monitor::Enter(m_objLockStatment);
	
	if (String::Compare(sCommand, "BYE", true) == 0)
	{
		OnDisconnect();
		return false;
	}

	cServerSIP::m_objServerSIP->OnTelnetCommand(sCommand, objParamList, this);

	Monitor::Exit(m_objLockStatment);
	
	return true;
}

void cTelnetClient::OnSuccessToAuth()
{
	SendText("\r\nSuccess!!\r\n\r\n");
	SendText("VaxTele:>");
}

void cTelnetClient::OnFailToAuth()
{
	SendText("\r\nFail!!\r\n\r\n");

	m_nAuthStep = AUTH_LOGIN;
	SendText("Login:");
}

void cTelnetClient::SendCommand(String^ sCommand, ArrayList^ objParamList)
{
	String^ sToData = sCommand;

	if(objParamList != nullptr)
	{
		for(int nCount = 0; nCount < objParamList->Count; nCount++)
			sToData = sToData + " " + objParamList[nCount];
	}

	sToData += "\r\n";
	m_objSocket->Send(Encoding::UTF8->GetBytes(sToData), 0, sToData->Length, SocketFlags::None);
}

void cTelnetClient::SendText(String^ sText)
{
	array<System::Byte>^ msg = Encoding::UTF8->GetBytes(sText);
	m_objSocket->Send(msg, 0, msg->Length, SocketFlags::None);
}

void cTelnetClient::OnAuthrization(String^ sData)
{
    if(m_nAuthStep == AUTH_LOGIN)
    {
        m_nAuthStep = AUTH_PASSWORD;
        m_sLoginTelnet = sData;

        SendText("Password:");
    }
    else if(m_nAuthStep == AUTH_PASSWORD)
    {
        m_nAuthStep = AUTH_DONE;

		if(cTelnetServer::m_objTelnetServer->IsAuth(m_sLoginTelnet, sData) == true)
            OnSuccessToAuth();
        else
            OnFailToAuth();
        
        m_sLoginTelnet = "";
    }
}

bool cTelnetClient::ReceiveData(String^ sRecvData)
{
    m_objTelnetRequest->FillRecvBuffer(sRecvData, this);

    while(true)
    {
        String^ sRequest = m_objTelnetRequest->FindRequest(this);

        if(sRequest->Length == 0)
            break;

        if (m_nAuthStep != AUTH_DONE)
        {
            OnAuthrization(sRequest);
            break;
        }
        
        String^ sCommand = gcnew String("");
        ArrayList^ objParamList = gcnew ArrayList();

        if(m_objTelnetRequest->ParseRequest(sRequest, sCommand, objParamList) == false)
            break;
        
        if(OnRequest(sCommand, objParamList) == false)
            return false;
    }

    return true;
}

void cTelnetClient::CloseSocket()
{
    Monitor::Enter(this);
    
    if (m_objSocket != nullptr)
    {
		m_objSocket->Close();
        m_objSocket = nullptr;
	}

	Monitor::Exit(this);
}