// CallEvent.cpp: implementation of the CCallEvent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ServerSIP.h"
#include "CallEvent.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define IN_BOUND_CALL_REJECT			0
#define IN_BOUND_CALL_CONNECT_USER		1
#define IN_BOUND_CALL_PUT_IN_QUEUE		2


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCallEvent::CCallEvent()
{

}

CCallEvent::~CCallEvent()
{

}

void CCallEvent::OnIncomingCallFromLine(CCallData* pCallData, CLineData* pLineData, LPCTSTR sDialNo) 
{
	if(pLineData->m_dwCallType == IN_BOUND_CALL_REJECT)
	{
		CServerSIP::m_pServerSIP->CloseCall(pCallData);
		return;
	}

	if(pLineData->m_dwCallType == IN_BOUND_CALL_CONNECT_USER)
	{
		CServerSIP::m_pServerSIP->DialCallToUser(pCallData, pLineData->m_sCallTo);
		return;
	}

	
	if(pLineData->m_dwCallType == IN_BOUND_CALL_PUT_IN_QUEUE)
	{
		CQueueData* pQueueData = CServerSIP::m_pServerSIP->m_objQueueList.FindQueue(pLineData->m_sCallTo);

		if(pQueueData == NULL)
		{
			CServerSIP::m_pServerSIP->CloseCall(pCallData);
		}
		else
		{
			pCallData->m_nCallType = CALL_TYPE_QUEUE;
			OnIncomingCallQueueType(pCallData, pQueueData->m_lQueueId);
		}
	}
	
}

void CCallEvent::OnIncomingCallFromUser(CCallData* pUserCallData, CUserData* pUserData, LPCTSTR sDialNo) 
{
	if(pUserData->m_bStealthListener)
	{
		pUserCallData->m_nCallType = CALL_TYPE_STEALTH;
		OnIncomingCallStealthType(pUserCallData, pUserData, sDialNo);
	}
	else
	{
		pUserCallData->m_nCallType = CALL_TYPE_REGULAR;
		OnIncomingCallRegularType(pUserCallData, pUserData, sDialNo);	
	}
}

//////////////////////////////////////////

void CCallEvent::OnIncomingCallQueueType(CCallData* pCallData, ULONG lQueueId)
{
	pCallData->m_lQueueId = lQueueId;
	CServerSIP::m_pServerSIP->AcceptCall(pCallData);
}

void CCallEvent::OnIncomingCallStealthType(CCallData* pCallData, CUserData* pUserData, LPCTSTR sDialNo)
{
	pCallData->m_sStealthDialNo = sDialNo;
	pCallData->m_sStealthDialNo.MakeLower();

	if(CServerSIP::m_pServerSIP->AcceptCall(pCallData))
		pUserData->m_lCallCount++;
}

void CCallEvent::OnIncomingCallRegularType(CCallData* pCallData, CUserData* pUserData, LPCTSTR sDialNo)
{
	int nDialPlanIndex = CServerSIP::m_pServerSIP->m_objDialPlanList.SearchDialPlan(sDialNo);
	
	if(nDialPlanIndex != -1)
	{
		CString sLineName;
		CString sPhoneNoToDial = CServerSIP::m_pServerSIP->m_objDialPlanList.ApplyDialPlan(nDialPlanIndex, sDialNo, sLineName);
		
		if(CServerSIP::m_pServerSIP->DialCallToLine(pCallData, sPhoneNoToDial, sLineName) != NULL)
			pUserData->m_lCallCount++;
	}
	else
	{
		if(CServerSIP::m_pServerSIP->DialCallToUser(pCallData, sDialNo) != NULL)
			pUserData->m_lCallCount++;
	}		
}

////////////////////////////////////////////////////

void CCallEvent::OnAcceptIncomingCallConnected(CCallData* pCallData) 
{
	pCallData->m_bConnected = TRUE;
	
	switch(pCallData->m_nCallType)
	{
		case CALL_TYPE_STEALTH:
			OnConnectedCallStealthType(pCallData);
			break;

		case CALL_TYPE_QUEUE:
			OnConnectedCallQueueType(pCallData);
			break;

		case CALL_TYPE_REGULAR:
			break;
	}
}

void CCallEvent::OnDialCallConnected(CCallData* pCallData) 
{
	pCallData->m_bConnected = TRUE;
							
	switch(pCallData->m_nCallType)
	{
		case CALL_TYPE_REGULAR:
			OnConnectedCallRegularType(pCallData);
			break;

		case CALL_TYPE_QUEUE:
			break;

		case CALL_TYPE_STEALTH:
			break;
	}
}

////////////////////////////////////////////////////////

void CCallEvent::OnConnectedCallRegularType(CCallData* pCallData)
{
	CCallData* pBindCallData = pCallData->m_pBindCallData;
		
	if(pBindCallData != NULL && pBindCallData->m_bConnected == FALSE)
	{
		if(!CServerSIP::m_pServerSIP->AcceptCall(pBindCallData))
		{
			CServerSIP::m_pServerSIP->CloseCall(pBindCallData);
            CServerSIP::m_pServerSIP->CloseCall(pCallData);

			return;
		}
	}
	
	CServerSIP::m_pServerSIP->StartVoiceStream(pCallData, pBindCallData);
	
	CServerSIP::m_pServerSIP->m_objStealthList.TryCaptureCall(pCallData);
	CServerSIP::m_pServerSIP->m_objStealthList.TryCaptureCall(pBindCallData);
}

void CCallEvent::OnConnectedCallStealthType(CCallData* pCallData)
{
	CServerSIP::m_pServerSIP->m_objStealthList.AddCallToStealthList(pCallData);

	long lStealthSessionId = CServerSIP::m_pServerSIP->m_objServerData.GetStealthSessionId();
	CServerSIP::m_pServerSIP->AddCallToSession(pCallData, lStealthSessionId);
}

void CCallEvent::OnConnectedCallQueueType(CCallData* pCallData)
{
	CServerSIP::m_pServerSIP->m_objQueueList.AddCallToQueue(pCallData);
}

////////////////////////////////////////////////////////

void CCallEvent::OnDialCallTimeOut(CCallData* pCallData) 
{
	CCallData* pBindCallData = pCallData->m_pBindCallData;

	if(pBindCallData != NULL)
		CServerSIP::m_pServerSIP->SendResponse(pBindCallData, 408, "Request Timeout");
	
	OnDisconnectCall(pCallData);
}

void CCallEvent::OnAcceptCallTimeOut(CCallData* pCallData)
{
	OnDisconnectCall(pCallData);
}

void CCallEvent::OnDisconnectCall(CCallData* pCallData) 
{
	CCallData* pBindCallData = pCallData->m_pBindCallData;
	
	if(pBindCallData != NULL)
		CServerSIP::m_pServerSIP->CloseCall(pBindCallData);
	
	CServerSIP::m_pServerSIP->CloseCall(pCallData);
}

void CCallEvent::OnIncomingCallCancel(CCallData* pCallData) 
{
	OnDisconnectCall(pCallData);
}

void CCallEvent::OnSessionLost(CCallData* pCallData) 
{
	OnDisconnectCall(pCallData);
}