using System;
using System.Collections;
using System.Text;
using System.Net.Sockets;
using System.Windows.Forms;
using System.Threading;
using System.IO;


namespace VaxTeleConfigSIP
{
    public class cTelnetClient
    {
        private const int AUTH_LOGIN = 100;
        private const int AUTH_PASSWORD = 101;
        private const int AUTH_WAIT = 102;
        private const int AUTH_DONE = 103;

        private const int AUTH_STATUS_SUCCESS = 103;
        private const int AUTH_STATUS_FAIL = 104;
        private const int AUTH_STATUS_WAIT = 105;

        private Socket m_objSocket = null;

        private string m_sLoginTelnet;
        private string m_sPasswordTelnet;
        private string m_sRecvBuffer;
        private int m_nAuthStep;
        private int m_nAuthStatus;

        private const int RECV_BUFFER_SIZE = 512;
        private byte[] m_sAsyncBuffer = new byte[RECV_BUFFER_SIZE];
                
        private void OnReceive(IAsyncResult ar)
        {
            lock (this)
            {
                if (m_objSocket == null) 
                    return;
                               
                try
                {
                    int nRead = m_objSocket.EndReceive(ar);

                    if (nRead == 0)
                    {
                        OnDisconnect();
                        return;
                    }

                    m_sRecvBuffer = m_sRecvBuffer + Encoding.ASCII.GetString(m_sAsyncBuffer, 0, nRead);
                    m_sRecvBuffer = m_sRecvBuffer.Replace("VaxTele:>", "");

                    ReceiveData();

                    if (m_nAuthStatus != AUTH_STATUS_FAIL)
                        m_objSocket.BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags.None, new AsyncCallback(OnReceive), 0);
                }
                catch (SocketException e)
                {
                    OnDisconnect();
                }
            }
        }

        private void ReceiveData()
        {
            while (true)
            {
                if (m_nAuthStep != AUTH_DONE)
                {
                    OnAuthrization(m_sRecvBuffer);
                    break;
                }

                string sRequest = FindRequest();
                if (sRequest.Length == 0) break;

                string sCommand = "";
                ArrayList objParamList = new ArrayList();

                if (ParseRequest(sRequest, ref sCommand, ref objParamList) == false)
                    break;
                                                
                OnRequest(sCommand, ref objParamList);
            }
        }

        private void OnRequest(string sCommand, ref ArrayList objParamList)
        {
            if (string.Compare(sCommand, "BYE", true) == 0)
            {
                OnDisconnect();
                return;
            }

            MainForm.m_objMainForm.InvokeTelnetRecv(sCommand, ref objParamList);
        }

        public Boolean Connect(string sToIP, int nToPort, string sLogin, string sPassword)
        {
            CloseSocket();
            
            m_objSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            
            m_objSocket.BeginConnect(sToIP, nToPort, null, 0);

            for (int nCount = 0; nCount < 30; nCount++)
            {
                Thread.Sleep(100);

                if (m_objSocket.Connected == true)
                    break;
            }

            if (m_objSocket.Connected == false)
            {
                MessageBox.Show("Fail to connect to server.", "Socket", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            m_sLoginTelnet = sLogin;
            m_sPasswordTelnet = sPassword;

            m_nAuthStep = AUTH_LOGIN;
            m_nAuthStatus = AUTH_STATUS_WAIT;
            m_sRecvBuffer = "";

            m_objSocket.BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags.None, new AsyncCallback(OnReceive), 0);

            for (int nCount = 0; nCount < 40; nCount++)
            {
                Thread.Sleep(100);

                if (m_nAuthStatus != AUTH_STATUS_WAIT)
                    break;
            }

            if (m_nAuthStatus != AUTH_STATUS_SUCCESS)
            {
                MessageBox.Show("Fail to connect to server.", "Socket", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            MessageBox.Show("Connected successfully.", "Socket", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            return true;
        }

        public string FindRequest()
        {
            int nStart = m_sRecvBuffer.IndexOf("\r\n");
            if (nStart == -1) return "";

            string sCommand = m_sRecvBuffer.Substring(0, nStart);
            m_sRecvBuffer = m_sRecvBuffer.Remove(0, nStart + 2);

            return sCommand;
        }

        public void OnAuthrization(string sData)
        {
            if (m_nAuthStep == AUTH_LOGIN)
            {
                if (sData.IndexOf("Login:") == -1)
                    return;

                m_nAuthStep = AUTH_PASSWORD;
                SendCommand(m_sLoginTelnet);

                m_sRecvBuffer = "";
            }
            else if (m_nAuthStep == AUTH_PASSWORD)
            {
                if (sData.IndexOf("Password:") == -1)
                    return;

                m_nAuthStep = AUTH_WAIT;
                SendCommand(m_sPasswordTelnet);

                m_sRecvBuffer = "";
            }
            else if (m_nAuthStep == AUTH_WAIT && sData.Length != 0)
            {
                m_sRecvBuffer = "";

                if (sData.IndexOf("Success") == -1)
                {
                    m_nAuthStep = AUTH_LOGIN;
                    m_nAuthStatus = AUTH_STATUS_FAIL;

                    return;
                }

                m_nAuthStep = AUTH_DONE;
                m_nAuthStatus = AUTH_STATUS_SUCCESS;
            }
        }

        private Boolean ParseRequest(string sRequest, ref string sCommand, ref ArrayList objParamList)
        {
            int nEnd = sRequest.IndexOf(" ");

            if(nEnd == -1)
            {
                sCommand = sRequest;
                return true;
            }

            sCommand = sRequest.Substring(0, nEnd);
            sRequest = sRequest.Remove(0, nEnd + 1);

            if(String.Compare(sCommand, "MSG", true) == 0 || String.Compare(sCommand, "StatusServerSIP", true) == 0 || String.Compare(sCommand, "ErrorLog", true) == 0 || String.Compare(sCommand, "StatusLog", true) == 0)
            {
                objParamList.Add(sRequest);
                return true;
            }

            while(true)
            {
                nEnd = sRequest.IndexOf(" ");

                if(nEnd == -1)
                {
                    objParamList.Add(sRequest);
                    break;
                }

                objParamList.Add(sRequest.Substring(0, nEnd));
                sRequest = sRequest.Remove(0, nEnd + 1);

            }

            return true;
        }

        public void SendCommand(string sCommand, ref ArrayList objParamList)
        {
            if (m_objSocket == null)
                return;
            
            string sToData = sCommand;

            for (int nCount = 0; nCount < objParamList.Count; nCount++)
                sToData = sToData + " " + objParamList[nCount];

            sToData += "\r\n";
            m_objSocket.Send(Encoding.UTF8.GetBytes(sToData), 0, sToData.Length, SocketFlags.None);
        }

        public void SendCommand(string sCommand)
        {
            if (m_objSocket == null)
                return;
            
            string sToData = sCommand + "\r\n";
            m_objSocket.Send(Encoding.UTF8.GetBytes(sToData), 0, sToData.Length, SocketFlags.None);
        }
    
        public void OnDisconnect()
        {
            CloseSocket();
            MessageBox.Show("Connection to host lost.", "Socket", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        public void CloseSocket()
        {
            if (m_objSocket != null)
            {
                lock (this)
                {
                    m_objSocket.Close();
                    m_objSocket = null;
                }
            }

            m_nAuthStatus = AUTH_STATUS_FAIL;
        }
    }

}
